
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will start a listensession from a BindStruct
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    BindToProtocol_c:
    StartProc 3

    mov rcx,retBindToProtocol_c
    xchg qword [rsp],rcx
    jmp BindToProtocol_asm

    retBindToProtocol_c:
    sub rsp,3*psize
    EndProc 3
    jmp rcx
else
    BindToProtocol_c:
end if
;---------
;asm implementation
;---------
BindToProtocol_asm:
call denter

;--mov pecx,endBindToProtocol
;--call
xor edx,edx

;-----------------------------
;denter needs to be called before jumping here
;EDX => unique Id
;-----------------------------
BindToProtocol_internal_jump:
plog LOG_MESSAGE,"Loading ListenSession '", 0x1, pdword [pebp+psize*4], "' with id '", 0x3, pedx, "'"

;if used LOG
;
;  mov pesi,pdword [pebp+psize*3] ;BindStruct
;  loopBinStructForLogging:
;
;
;end if

push pdword [pebp+psize*4] ;name
push pedx                  ;unique Id
push pdword [pebp+psize*3] ;BindTo param
call CreateInternalListenStructure
test peax,peax
jz endBindToProtocol
mov pebx,peax


push pdword [pebx+psize]
call copyListenSession
mov pedx,peax ;External ListenSession on (R/E)DX

;Add the internal Listen Structure to the listed, indexed on the ExternalListenSession
push pebx             ;Item   (internal lsess)
push pedx             ;Index  (public lsess)
push [listenSessions] ;Collection
call AddIndexedDwordItem ;AddIndexedItem
xor eax,eax ;Clear Item ID

mov eax,dword [pebx+(5*psize)+4]
push peax
call releaseHighestListenSessionsId

;Get BindedTohandle from the InternalListenStructure
mov pebx,pdword [pebx]
;mov pebx,pdword [pebx]

;get internal protocol session
push 1
push pebx
call GetDwordItem
mov pebx,peax   ;external psess

push peax
call getInternalProtocolStructure

;get modstruct
if used is64
  mov r12,pdword [peax+psize*4]
else
  mov pedi,pdword [peax+psize*4]
end if
mov pesi,pedx                   ;Save the lsess

MakeStack

if used is64
  if used isLin
    call denter

    argpush 2,pedx                       ;lsess
    argpush 3,0                          ;flags
    argpush 1,pebx                       ;psess
  end if
end if

if used is32 | used isWin
  ;Make call to first BindToProtocolCallBack
  argpush 3,0                          ;flags
  argpush 2,pedx                       ;lsess
  argpush 1,pebx                       ;psess
end if

if used is32
  call pdword [pedi+(psize*(2+BindToProtocolCallBack))]
else
  call pdword [r12+(psize*(2+BindToProtocolCallBack))]
end if

if used is64
  if used isLin
    call dleave
  end if
end if
ClearStackAftherCall 3

test eax,eax ;save retv
jz errBindToProtocol

;find internal ListenSession
push pesi                  ;name (lsess)
push [listenSessions]      ;collectiom
call GetIndexedDwordItem
mov pebx,peax              ;internal listensession

push pebx                ;interal listensession
push pesi                ;lsess
call SaveListenSessionToDisk

cmp dword [pebx+POS_STATE_INTERNALLISTENSESSION],LSESS_STATE_ERROR
je errBindToProtocol

;set state loaded
mov dword [pebx+POS_STATE_INTERNALLISTENSESSION],LSESS_STATE_LOADED

push pesi                ;lsess (param)
ppush callStartListening  ;baseAddress
call createthread

;argpush 1,pesi
;call callStartListening

;--
if used is64 & used isWin
  fixStack 0    ;align stack on 0x10  (win x64 calling convention)
  sub pesp,0x10 ;create shadow places for the function call (for windows x64)
end if

;call the registerd ListenSessionIsCreated functions
push [registerdListenSessionIsCreatedFunctions]
call CountItems
mov ecx,eax

jecxz endLoopListenSessionIsCreatedCallbackFunctions ;jump when there are no callback functions registerd

loopListenSessionIsCreatedCallbackFunctions:
  push pecx
  push [registerdListenSessionIsCreatedFunctions]
  call GetDwordItem
  test peax,peax
  jz endLoopListenSessionIsCreatedCallbackFunctions

  mov pebx,pecx ;save ecx

  if used is64
    if used isLin
      mov pecx,peax
      call denter

      argpush 1,pesi ;lsess
      argpush 2,0 ;no flags

      call pecx ;call ListenSessionIsCreated Callback Function
    end if
  end if

  if used is32 | used isWin
    argpush 2,0 ;no flags
    argpush 1,pesi ;lsess

  call peax ;call ListenSessionIsCreated Callback Function
  end if

  if used is64 & used isLin
      call dleave
  end if
  ClearStackAftherCall 2


  mov pecx,pebx ;restore ecx

nextLoopListenSessionIsCreatedCallbackFunctions:
  loop loopListenSessionIsCreatedCallbackFunctions
endLoopListenSessionIsCreatedCallbackFunctions:
;argpush 1,pesi                ;lsess (param)
;call peax                ;baseAddress
;--mov dword [pebx+POS_THREADID_INTERNALLISTENSESSION],peax

endBindToProtocol:
call dleave
ret psize*3

errBindToProtocol:
;Delete listensession from the listenSessions list
push pesi                  ;name (lsess)
push [listenSessions]      ;collectiom
call RemoveDwordByIndex

;remove InternalListenStructure
push peax
call FreeInternalListenStructure

;remove ListenSession
push pesi
call FreeListenSession

xor eax,eax
jmp endBindToProtocol

;------------------------------------------------------------------------------
;The RequestToBindToNextProtocol function will bind the listensession to the next protocol configured with function BindToProtocol.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RequestToBindToNextProtocol_c:
    StartProc 2

    mov rcx,retRequestToBindToNextProtocol_c
    xchg qword [rsp],rcx
    jmp RequestToBindToNextProtocol_asm

    retRequestToBindToNextProtocol_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    RequestToBindToNextProtocol_c:
end if
;---------
;asm implementation
;---------
RequestToBindToNextProtocol_asm:
call denter

mov pecx,errRequestToBindToNextProtocol
call checkProtocolSession

mov pecx,errRequestToBindToNextProtocol
call checkListenSession
mov pedi,pdword [peax] ;BindedTohandle

;get next psess and get the modstruct of this psess
xor ecx,ecx ;increment psess
call FindNextOrPreviousPsess
test eax,eax
jz endRequestToBindToNextProtocol

dec eax
test eax,eax
jnz endRequestToBindToNextProtocol


MakeStack

if used is64
  if used isLin
    ;Make call to first BindToProtocolCallBack
    argpush 3,0                          ;flags
    argpush 1,pesi                       ;psess
    argpush 2,pdword [pebp+psize*3]      ;lsess
  end if
end if

if used is32 | used isWin
  ;Make call to first BindToProtocolCallBack
  argpush 3,0                          ;flags
  argpush 2,pdword [pebp+psize*3]      ;lsess
  argpush 1,pesi                       ;psess
end if



if used is64
  call pdword [r12+(psize*(2+BindToProtocolCallBack))]
else
  call pdword [pedx+(psize*(2+BindToProtocolCallBack))]
end if
ClearStackAftherCall 3
test eax,eax
jnz errRequestToBindToNextProtocol

;set listensession to error state
mov pecx,errRequestToBindToNextProtocol
call checkListenSession
mov dword [peax+POS_STATE_INTERNALLISTENSESSION],LSESS_STATE_ERROR

jmp errRequestToBindToNextProtocol

endRequestToBindToNextProtocol:
errRequestToBindToNextProtocol:
call dleave
ret psize*2