include 'include/flags.inc'

NewLineA equ 0x0D,0x0A
NewLineStringA db NewLineA,0
none db 'None',0
;--unknown db 'unknown',0
if used isWin
  unknownW du 'unknown',0
else
  unknownW db 'unknown',0
end if

;LOG_MESSAGE  0x1 level1: 0x1 message
;LOG_WARNING 0x2 level2: 0x2 warning
;LOG_ERROR 0x4 level3: 0x4 error

allowedLog dd LOG_MESSAGE or LOG_WARNING or LOG_ERROR

macro CompileLogMessage
{ common
  virtual at 0
    call LOG
    LOG:
  end virtual
}

log_location pointer 0
logp pointer 0

macro startLog
{ common
if used isLin
  mov peax,[log_location]
  cmp word [peax],0x31
  jne dontUseStdOut

    mov [logp],1
    jmp donLoadingLogOut

  dontUseStdOut:
end if
push 0x1 or 0x2 or 0x4 or 0x10000 or 0x20000 ;WRITE&READ or CREATE_FILE or SHARE_READ_WHILE_WRITE or filename is UTF-8 OR don't make long path
push [log_location]
call openFileHandle
cmp eax,-1
je LogfileNotOpend

LogfileOpend:
mov [logp],peax

LogfileNotOpend:
push peax
call seekToEndPointerFile

donLoadingLogOut:
}

;old levels:
;level 1: starting application
;level 2: Windows service
;level 3: Starting TV's server
;level 4: Error TV's server
;level 20: Other modules

;new levels:
;level1: 0x1 message
;level2: 0x2 warning
;level3: 0x4 error

;flags:
;0x10000 welcome log (must always be printed)

macro log level, message
{ common
local ..isNotWritingLogToOut
local ..fileOutIsPushed
local ..isNotError
if used LOG | ((level and LOG_WELCOME_MESSAGE) = LOG_WELCOME_MESSAGE)
  local ..endLog
  local ..endLogWithPop
  cmp [logp],0
  je ..endLog

  local ..logMessage
  local ..logMessageBegin

  push peax ;save return value

  if ~((level and LOG_WELCOME_MESSAGE) = LOG_WELCOME_MESSAGE)
    push peax
    mov eax,(level and 0xFFFF) ;level
    and eax,[allowedLog]
    pop peax
    jz ..endLogWithPop  ;log not allowed
  end if

  push ..logMessage-..logMessageBegin
  call ..logMessage
  ..logMessageBegin:
  db message
  db 0x0D,0x0A
  ..logMessage:
  if used isLin
    cmp [logp],1
    jne ..isNotWritingLogToOut

      push peax
      mov eax,LOG_ERROR
      test eax,(level and 0xFFFF) ;level
      jz ..isNotError

        pop peax
        push 2 ;stdErr
        jmp ..fileOutIsPushed

    ..isNotError:
      pop peax
    ..isNotWritingLogToOut:
  end if
  push [logp]
  ..fileOutIsPushed:
  call writefile

  ..endLogWithPop:
  pop peax ;restore return value
  ..endLog:
end if
}

macro logmod level, protocolSession, listenSession, clientSession, moduleNumber, message
{ common
if used LOG
  local ..logMessage
  local ..logMessageBegin
  local ..logFunctionUnknown

  cmp [Log],0
  je ..logFunctionUnknown

  push peax ;Save return value

  push level
  call ..logMessage
  ..logMessageBegin:
  db message
  db 0
  ..logMessage:
  ppush moduleNumber
  ppush clientSession
  ppush listenSession
  ppush protocolSession
  call [Log]

  pop peax ;restore return value

  ..logFunctionUnknown:
end if
}

macro plogmod level, protocolSession, listenSession, clientSession, moduleNumber, [message]
{ common
if used LOG
   local ..noLogFileOpenForWrite
   if ~ message eq
     local ..newline
    local ..logFunctionUnknown

    cmp [Log],0
    je ..logFunctionUnknown

     push peax ;Save return value

     push 0   ;last item

     reverse
       if message eqtype ""
         local ..logMessage

         call ..logMessage
         db message
         db 0
         ..logMessage:
         push 0x1 ;ASCII string
       else
         ppush message
       end if
     common

     push 0 ;output string (CreateOneStringUtf8 creates new buffer)
     push 0 ;flags
     call CreateOneStringUtf8 ;--CreateOneStringA
     push peax ; push for freeeing memory
     ;push peax ; push to save pointer
     ;push peax ; push for strlen
     ;call strlen
     ;xchg peax,pdword [pesp] ;push length for writefile

     push level
     push peax ;push pointer for Log

     ppush moduleNumber
     ppush clientSession
     ppush listenSession
     ppush protocolSession
     call [Log]

     call FreeMemLocation; free string created with CreateOneStringA
     pop peax ;Restore return value

     ..logFunctionUnknown:
   end if
end if
}

macro plog level, [message]
{ common
local ..isNotWritingLogToOut
local ..fileOutIsPushed
local ..isNotError

printCommands = 1
if (level eqtype 0)
  printCommands = 0
  if ((level and LOG_WELCOME_MESSAGE) = LOG_WELCOME_MESSAGE) | used LOG
    printCommands = 1
  end if
end if

if ((level eqtype 0 & printCommands = 1) | ~ level eqtype 0)
   local ..noLogFileOpenForWrite
    cmp [logp],0
    je ..noLogFileOpenForWrite

   if ~ message eq
     local ..newline
     local ..endLog

     push peax ;save return value

    if (level eqtype 0)
      if ~ ((level and LOG_WELCOME_MESSAGE) = LOG_WELCOME_MESSAGE)
        push peax
        mov eax,(level and 0xFFFF) ;level
        and eax,[allowedLog]
        pop peax
        jz ..endLog  ;log not allowed
      end if
    end if

     push 0   ;last item
     call ..newline
       db 0x0D,0x0A,0
     ..newline:
     push 0x1 ;ASCII string

     reverse
       if message eqtype ""
         local ..logMessage

         call ..logMessage
         db message
         db 0
         ..logMessage:
         push 0x1 ;ASCII string
       else
         ppush message
       end if
     common

     push 0 ;output string (CreateOneStringUtf8 creates new buffer)
     push 0 ;flags
     call CreateOneStringUtf8 ;--CreateOneStringA
     push peax ; push for freeeing memory
     push peax ; push to save pointer
     push peax ; push for strlen
     call strlen
     xchg peax,pdword [pesp] ;push length for writefile
     push peax ;push pointer for write file

     if used isLin
       cmp [logp],1
       jne ..isNotWritingLogToOut

         push peax
         mov eax,level
         and eax,0xFFFF
         test eax,LOG_ERROR ;level
         jz ..isNotError

           pop peax
           push 2 ;stdErr
           jmp ..fileOutIsPushed

       ..isNotError:
         pop peax
       ..isNotWritingLogToOut:
     end if
     push [logp]
     ..fileOutIsPushed:
     call writefile

     call FreeMemLocation; free string created with CreateOneStringA

     ..endLog:
     pop peax ;restore return value
   end if
   ..noLogFileOpenForWrite:
end if
}


if used Log_asm
;------------------------------------------------------------------------------
;This function registers log generated by the plug-ins
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    Log_c:
    StartProc 6

    mov rcx,retLog_c
    xchg qword [rsp],rcx
    jmp Log_asm

    retLog_c:
    sub rsp,6*psize
    EndProc 6
    jmp rcx
else
    Log_c:
end if
  ;---------
  ;ASM implementation
  ;---------
  ;ProtocolSession, Listensession, ClientSession, Module, message, level
  Log_asm:
  call denter

  cmp [logp],0
  je endLog

  and dword [pebp+psize*7],0xFFFF ;remove flag WELCOME_LOG
  mov eax,dword [pebp+psize*7] ;level
  and eax,[allowedLog]
  jz endLog  ;log not allowed

  test eax,eax
  jz endLog  ;log not allowed

  mov pecx,noProtocolSession
  call checkProtocolSession
  mov pedx,peax ;(R/E)DX => Internal ProtocolSession
  jmp ProtocolSessionIsFound

  noProtocolSession:
  xor pedx,pedx

  ProtocolSessionIsFound:
  mov pebx,pdword [pebp+psize*5] ;(R/E)BX => Module
  mov pecx,pdword [pebp+psize*3] ;(R/E)CX => Listensession
  mov pesi,pdword [pebp+psize*4] ;(R/E)SI => ClientSession

  test pesi,pesi
  jz noClientSession

    ;Get ListenSession from ClientSession
    push pesi
    push 0
    call GetListenSessionFromClientSession_internal_jump
    mov pecx,peax

  noClientSession:
  jpcxz noListenSession

    ;get internal ListenSession
    mov pdword [pebp+psize*3],pecx
    mov pecx,errListenSessionCheck
    call checkListenSession
    mov pecx,pdword [peax+8+6*psize] ;(R/E)CX => Name of listenSession
    jmp noListenSession

  errListenSessionCheck:
  xor ecx,ecx

  noListenSession:
  test pedx,pedx
  jz noProtocolSessionForLog

    ;mov pedi,pdword [pedx+(psize*4)] ;Get module structure
    ;mov pedi,pdword [pedi+(psize*1)] ;get HModule  (set temp on R/EDI)
    ;test pebx,pebx
    ;cmovz pebx,pedi  ;No module was passed to Log, use the one from ProtocolSession

    mov pedx,pdword [pedx+psize*2] ;copyPsess
    mov pedx,pdword [pedx+psize]   ;protocolName


  noProtocolSessionForLog:

  push 0 ;push for FreeModuleName

  test pebx,pebx
  jz noModuleHandle
    push pebx
    call GetModuleName
    mov pdword [pesp],peax ;push for FreeModuleName

    push peax
    call GetFilenameFromPath
    mov pebx,peax ;EBX= modulename

  noModuleHandle:
  mov peax,none

  mov pedi,pdword [pebp+psize*6] ;message

  test pedi,pedi
  cmovz pedi,peax

  ;0x0=last item; 0x1=ANSI string; 0x11=UTF-8 string 0x2=UTF-16 string;; 0x22=Multi UTF-16 string
  ;               0x3=Decimal number; 0x31=Hexidecimal number;
  ;               0x4=decimal 64 bit number (pointer in x86, same as 3 in x64);
  ;               0x41=Hexidecimal number 64 bit number (pointer in x86, same as 31 in x64);
  ;               0x5=Pointer to long long that needs to formated to "5GB (5.000.000.000 bytes)"
  ;               0xA=Begin JSON object, 0xB=end JSON object, 0xE=Boolean, 0xF=begin JSON array
  ;               0x10=end JSON array

  test pesi,pesi
  jz noClientSessionToLog

  test pebx,pebx
  jz noModuleToDesplay1
    plog dword [pebp+psize*7],"[",moduleNameType,pebx,"] (sess: ",0x31,pesi,",  lsess: ",0x11,pecx, ", psess: ",0x11,pedx,"): ",0x11,pedi
    jmp doneLogging
  noModuleToDesplay1:
    plog dword [pebp+psize*7],"[sess: ",0x31,pesi,",  lsess: ",0x11,pecx, ", psess: ",0x11,pedx,"] ",0x11,pedi
    jmp doneLogging

noClientSessionToLog:
  test pecx,pecx
  jz noListenSessionToLog

  test pebx,pebx
  jz noModuleToDesplay2
    plog dword [pebp+psize*7],"[",moduleNameType,pebx,"] (lsess: ",0x11,pecx,",  psess: ",0x11,pedx,"): ",0x11,pedi
    jmp doneLogging
  noModuleToDesplay2:
    plog dword [pebp+psize*7],"[lsess: ",0x11,pecx,",  psess: ",0x11,pedx,"] ",0x11,pedi
    jmp doneLogging

noListenSessionToLog:
  test pedx,pedx
  jz noProtocolSessionToLog

  test pebx,pebx
  jz noModuleToDesplay3
    plog dword [pebp+psize*7],"[",moduleNameType,pebx,"] (psess: ",0x11,pedx,"): ",0x11,pedi
    jmp doneLogging
  noModuleToDesplay3:
    plog dword [pebp+psize*7],"[psess: ",0x11,pedx,"] ",0x11,pedi
    jmp doneLogging

noProtocolSessionToLog:
  test pebx,pebx
  jnz moduleToLog

  mov pebx,unknownW

  moduleToLog:
  plog dword [pebp+psize*7],"[",moduleNameType,pebx,"] ",0x11,pedi

doneLogging:
                  ;0x1,NewLineStringA
  call FreeModuleName

endLog:
  call dleave
  ret psize*6
end if