
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will start a clientsession
;------------------------------------------------------------------------------
StartClientSession:
StartProc 1
call denter
MakeStack

;find internal ListenSession
push pdword [pebp+psize*2] ;name (sess)
push [clientSessions]      ;collectiom
call GetIndexedDwordItem
mov peax,pdword [peax]     ;BindedTohandle

;get first psess of binded protocols
push 1
push peax
call GetDwordItem
mov pebx,peax              ;set psess on (R/E)BX

;get internal protocol session
push peax
call getInternalProtocolStructure
test peax,peax
jz endcallStartListening

;get modstruct
mov peax,pdword [peax+psize*4]

;Make call to StartListening
argpush 2,pdword [pebp+psize*2]      ;sess
argpush 1,pebx                       ;psess
call pdword [peax+(psize*(2+HandleConnection))]

ClearStackAftherCall 2

push pdword [pebp+psize*2] ;name (sess)
push [clientSessions]      ;collection
call RemoveDwordByIndex

push peax
call FreeInternalClientStructure

push pdword [pebp+psize*2]
call FreeClientSession




call dleave
EndProc 1
;--endCreatedThread
ret (FunctionReturnFactor*1)


;------------------------------------------------------------------------------
;The GetListenSessionFromClientSession function will return the corrosponding ListenSession from a ClientSession
;------------------------------------------------------------------------------
;---------
;Internal call, skip protocolsession check
;---------
GetListenSessionFromClientSession_internal:
call denter
jmp GetListenSessionFromClientSession_internal_jump
;---------
;C implementation
;---------
if used is64
    GetListenSessionFromClientSession_c:
    StartProc 2

    mov rcx,retGetListenSessionFromClientSession_c
    xchg qword [rsp],rcx
    jmp GetListenSessionFromClientSession_asm

    retGetListenSessionFromClientSession_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    GetListenSessionFromClientSession_c:
end if
;---------
;asm implementation
;---------
GetListenSessionFromClientSession_asm:
call denter

mov pecx,errGetListenSessionFromClientSession
call checkProtocolSession

GetListenSessionFromClientSession_internal_jump:

mov pecx,errGetListenSessionFromClientSession
call checkClientSession

mov peax,pdword [peax+psize*1] ; bindedToLsess

endGetListenSessionFromClientSession:
errGetListenSessionFromClientSession:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The CallNextHandleConnection function will call the HandleConnection function from the next binded psess
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    CallNextHandleConnection_c:
    StartProc 2

    mov rcx,retCallNextHandleConnection_c
    xchg qword [rsp],rcx
    jmp CallNextHandleConnection_asm

    retCallNextHandleConnection_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    CallNextHandleConnection_c:
end if
;---------
;asm implementation
;---------
CallNextHandleConnection_asm:
call denter

mov pecx,errCallNextHandleConnection
call checkProtocolSession

mov pecx,errCallNextHandleConnection
call checkClientSession
mov pebx,peax ;(R/E)BX = internal clientsession

xor ecx,ecx ;increment psess (param of FindNextOrPreviousPsess)

;get next psess and get the modstruct of this psess
mov pedx,pdword [pebp+psize*2]
call FindNextOrPreviousPsessEx

;return if error
test eax,eax
jz endCallNextHandleConnection

;return if no more psess
dec eax
test eax,eax
jnz endCallNextHandleConnection

cmp word [pedi+(psize*7)+4],PT_LABORER
jne callHandleConnection

call DecideLaborerProtocol

callHandleConnection:
xor edi,edi     ;Clear the internal protocolstructure
xor ebx,ebx     ;Clear internal ListenStructure
MakeStack

if used isLin
  if used is64
    argpush 1,pesi                       ;psess
    argpush 2,pdword [pebp+psize*3]      ;sess
  end if
end if

;Make call to first BindToProtocolCallBack
if used is32 | used isWin
  argpush 2,pdword [pebp+psize*3]      ;sess
  argpush 1,pesi                       ;psess
end if

if used is64
  call pdword [r12+(psize*(2+HandleConnection))]
else
  call pdword [pedx+(psize*(2+HandleConnection))]
end if

ClearStackAftherCall 2

endCallNextHandleConnection:
errCallNextHandleConnection:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The DecideLaborerProtocl function decides which Protocol should be activated (if there are more than one LABORER protocols) and is called from inside CallNextHandleConnection
;------------------------------------------------------------------------------
DecideLaborerProtocol:
call denter

;Test is there are more LABORER protocols
mov peax,pebx ;internal ClientStructure
xor ecx,ecx   ;increment psess
mov pedx,pesi ;Set psess on (R/E)DX
call FindNextOrPreviousPsessEx

;There is only one Laborer protocol, return
dec eax
test eax,eax
jnz endDecideLaborerProtocol

call dleave

;Create a temp collection to store psess, sign & modstructs
call CreateCollection
mov pebx,peax

mov pedx,pecx ;BindedToHandle

;Lock the list so no other threads can use it
push FULL_LOCK
push pedx
call LockListAcces

;find the psess in the BindedTohandle list
push pdword [pebp+psize*2] ;psess
push pedx                  ;BindedToHandle Collection
call FindDword
mov ecx,eax
inc ecx                    ;increment to next psess

xor edi,edi ;The count of the bigest signature

;r12 or dword [esp]   =  protocol with NO signature
;r13 or dowrd [esp+4] =  protocol protocol currently active in loop CollectProtocolSignatures
if used is32
  push 0
  push 0
else
  xor r12,r12
end if

;Collect psess, sign & modstruct of the binded protocols
loopCollectProtocolSignatures:
  ;get psess of the next binded protocol list
  push pecx
  push pedx
  call GetDwordItem
  test peax,peax
  jz endLoopCollectProtocolSignatures

  ;save psess on dword [esp] or r13
  if used is64
    mov r13,rax
  else
    mov dword [pesp+4],eax
  end if

  push peax ;Pre-push (R/E)AX for function 'getInternalProtocolStructure'

  ;push psess on the temp collection
  push peax
  push pebx
  call AddDword

  ;get internal protocol session
  ;push peax     ;psess already pushed
  call getInternalProtocolStructure

  push pdword [peax+(psize*7)+0x10+(psize*2)]  ;Pre-push Signature for later use
  push pdword [peax+(psize*7)+0x10+(psize*2)]  ;Pre-push Signature for function 'AddDword'

  ;push modstruct on the temp collection
  push pdword [peax+psize*4]
  push pebx
  call AddDword

  ;push Signature on the temp collection
  ;push push pdword [peax+psize*4]    ;psess already pushed
  push pebx
  call AddDword

  pop peax
  push pesi ;save ESI
  mov pesi,peax ;(R/E)SI = Signatures struct
  test pesi,pesi
  if used is64
    ;protocol with NO signature = protocol protocol currently active in loop CollectProtocolSignatures
    cmovz r12,r13  ;move psess of
    jz noSignatureInThisBindedProtocol
  else
    jnz getBigestSignature

    ;protocol with NO signature = protocol protocol currently active in loop CollectProtocolSignatures
    mov eax,dword [pesp+8]
    mov dword [pesp+4],eax
    jmp noSignatureInThisBindedProtocol
  end if


  getBigestSignature:
    lodsd ;get size member
    test eax,eax
    jz noSignatureInThisBindedProtocol

    cmp eax,edi
    cmova edi,eax ;move if EAX > edi

    add pesi,peax
  jmp getBigestSignature

  noSignatureInThisBindedProtocol:
  pop pesi ;(R/E)SI = psess of first Laborer protocol

  inc ecx
jmp loopCollectProtocolSignatures

endLoopCollectProtocolSignatures:
;EDI                = size bigest signature
;(R/E)SI            = psess of first Laborer protocol (needed for RecvData_asm)
;(R/E)DX            = BindedToHandle
;(R/E)BX            = collection with psess, modstruct and signatures of BindedToHandle
;R12 or dword [esp] = psess of protocol with no signature

;ask for a buffer where that fits the bigest signature
push pedi
call GetMemLocation
xchg peax,pedi  ;(R/E)DI = buffer, (R/E)AX = size biggest signature

;push size bigest signature on stack (for len param)
push peax

push 1 ;first get 1 byte

;call RecvData to wait 5 seconds a the client to respond (send 1 byte)
push pesp                                       ;&len
push pedi                                       ;buf
push 5000                                       ;Client got 5 seconds to respond
push RSF_RETURN_ON_COMPLETE or RSF_PEEK_DATA    ;Flag, peek data (dont remove from internal buffer) and return when buf is full or Timeout is rechead
push pdword [pebp+psize*3]                      ;Listensession
push pesi                                       ;Protocolsession (of the first Laborer protocol)
call RecvData_asm

add pesp,psize ;delete the '1' byte

;call RecvData to get the signature of the client (max waittime is 1/2 second)
push pesp                                       ;&len
push pedi                                       ;buf
push 1000                                       ;Client got 5/10 seconds to respond
push RSF_RETURN_ON_COMPLETE or RSF_PEEK_DATA    ;Flag, peek data (dont remove from internal buffer) and return when buf is full or Timeout is rechead
push pdword [pebp+psize*3]                      ;Listensession
push pesi                                       ;Protocolsession (of the first Laborer protocol)
call RecvData_asm


pop pecx

test eax,eax ;CE_NOERR
jz ClientSendSignature

jecxz noDataReceived

cmp eax,CE_TIMEOUT ;if if a timeout occured, it still could be a signature
jne noDataReceived

ClientSendSignature:
;remove psess of protocol with no
if used is32
  add pesp,0x8
end if

xor ecx,ecx ;index
inc ecx     ;index += 1

compaireSignatures:
  add ecx,2

  ;get signature
  push pecx
  push pebx
  call GetDwordItem
  test peax,peax
  jz endCompaireSignatures
  mov pesi,peax

  loopSignatures_compair:
    lodsd
    test peax,peax
    jz endLoopSignatures_compair
    if used is32
      push eax
    else
      mov r12,rax
    end if

    ;check if the signature match
    push peax
    push pesi
    push pedi
    call comapairMemCaseSen
;    test eax,eax
    jz SignatureFound

    ;next signature
    if used is32
      pop eax
      add esi,eax
    else
      add rsi,r12
    end if
  jmp loopSignatures_compair

  endLoopSignatures_compair:
  inc ecx ;next protocol
jmp compaireSignatures

endCompaireSignatures:
;
;end function (since no protocol can handle the connection)
;
push pedi
call FreeMemLocation

push pebx
call FreeDwordCollection

;--Unlock:
push FULL_LOCK
push pedx
call ExternalUnlockListAccesEx

if used isLinux
  mov peax,errCallNextHandleConnection
  mov pdword [pesp],peax
else
  pmov pdword [pesp],errCallNextHandleConnection
end if
xor eax,eax
ret

SignatureFound:
if used is32
  add pesp,0x4
end if
;index of the psess to activate
sub ecx,2
;get psess of the protocol to activate in this ClientSession
push pecx
push pebx
call GetDwordItem
mov pesi,peax ;(R/E)SI = psess to activate
jmp activateFoundPsess

noDataReceived:
;(R/E)SI = psess that need to be activated
if used is32
  pop pesi ;(R/E)SI = psess to activate
  add pesp,0x4
else
  mov rsi,r12
end if

;if no protocol with no signature, end connection
test pesi,pesi
jz endCompaireSignatures

activateFoundPsess:
;free the buffer
push pedi
call FreeMemLocation

xor ecx,ecx ;counter
inc ecx
xor edi,edi ;index of the protocol to activate

;Delete the other psess'es from BindedToHandle
loopDeleteNotNeededProtocol:
  ;get psess from list
  push pecx
  push pebx
  call GetDwordItem
  test peax,peax
  jz psessDeletedFromBindedToHandle

  cmp peax,pesi
  cmovz edi,ecx  ;set index of protocol to activate
  je DontDeleteThisPsess

  ;find the index of the psess in the BindedToHandle list and remove it
  push peax
  push pedx
  call FindDword

  push peax
  push pedx
  call RemoveDwordItem

  DontDeleteThisPsess:
  add ecx,3
jmp loopDeleteNotNeededProtocol

psessDeletedFromBindedToHandle:
;get psess from list
push pedi
push pebx
call GetDwordItem
mov pesi,peax

inc edi

;get modstruc from list
push pedi
push pebx
call GetDwordItem
if used is32
  mov pedx,peax
else
  mov r12,peax
end if

push pebx
call FreeDwordCollection

;--Unlock:
push FULL_LOCK
push pedx
call ExternalUnlockListAccesEx

ret ;denter is not called

endDecideLaborerProtocol:
call dleave
ret

;------------------------------------------------------------------------------
;The SendData function prepares and sends data to the client.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    SendData_c:
    StartProc 6

    mov rcx,retSendData_c
    xchg qword [rsp],rcx
    jmp SendData_asm

    retSendData_c:
    sub rsp,6*psize
    EndProc 6
    jmp rcx
else
    SendData_c:
end if
;---------
;asm implementation
;---------
SendData_asm:
call denter

SendData_next_psess:
mov pecx,errSendData
call checkProtocolSession

mov pecx,errSendData
call checkClientSession

xor ecx,ecx
inc ecx         ;decrement psess

;get next psess and get the modstruct of this psess
call FindNextOrPreviousPsess
test eax,eax
jz endSendData

dec eax
test eax,eax
jnz endSendData

if used is64
  cmp pdword [r12+(psize*(2+SendData))],0
else
  cmp pdword [pedx+(psize*(2+SendData))],0
end if
jne callNextSendData

;The current protocol does not export SendData, try the next psess
mov pdword [pebp+psize*2],pesi
jmp SendData_next_psess

callNextSendData:
MakeStack

;Make call to first BindToProtocolCallBack
mov peax,pdword [pebp+psize*7]
if used isLin
  if used is64
    argpush 6,peax                       ;*len
     mov peax,pdword [pebp+psize*6]
    argpush 5,peax                       ;buf
    argpush 4,pdword [pebp+psize*5]      ;timeout
    argpush 3,pdword [pebp+psize*4]      ;flags
    argpush 1,pesi                       ;psess
    argpush 2,pdword [pebp+psize*3]      ;sess
  end if
end if

if used is32 | used isWin
  argpush 6,peax                       ;*len
  mov peax,pdword [pebp+psize*6]
  argpush 5,peax                       ;buf
  argpush 4,pdword [pebp+psize*5]      ;timeout
  argpush 3,pdword [pebp+psize*4]      ;flags
  argpush 2,pdword [pebp+psize*3]      ;sess
  argpush 1,pesi                       ;psess
end if

if used is64
  call pdword [r12+(psize*(2+SendData))]
else
  call pdword [pedx+(psize*(2+SendData))]
end if
ClearStackAftherCall 6

endSendData:
errSendData:
call dleave
ret psize*6

;------------------------------------------------------------------------------
;The RecvData function prepares and sends data to the client.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RecvData_c:
    StartProc 6

    mov rcx,retRecvData_c
    xchg qword [rsp],rcx
    jmp RecvData_asm

    retRecvData_c:
    sub rsp,6*psize
    EndProc 6
    jmp rcx
else
    RecvData_c:
end if
;---------
;asm implementation
;---------
RecvData_asm:
call denter

RecvData_next_psess:
mov pecx,errRecvData
call checkProtocolSession

mov pecx,errRecvData
call checkClientSession

xor ecx,ecx
inc ecx         ;decrement psess

;get next psess and get the modstruct of this psess
call FindNextOrPreviousPsess
test eax,eax
jz endRecvData

dec eax
test eax,eax
jnz endRecvData

if used is64
  cmp pdword [r12+(psize*(2+RecvData))],0
else
  cmp pdword [pedx+(psize*(2+RecvData))],0
end if
jne callNextRecvData

;The current protocol does not export RecvData, try the next psess
mov pdword [pebp+psize*2],pesi
jmp RecvData_next_psess

callNextRecvData:

MakeStack

if used is64 & used isLin
    mov peax,pdword [pebp+psize*7]
    argpush 6,peax                       ;*len
    mov peax,pdword [pebp+psize*6]
    argpush 5,peax                       ;buf
    argpush 4,pdword [pebp+psize*5]      ;timeout
    argpush 3,pdword [pebp+psize*4]      ;flags
    argpush 1,pesi                       ;psess
    argpush 2,pdword [pebp+psize*3]      ;sess
end if

;Make call to first BindToProtocolCallBack
if used is32 | used isWin
  mov peax,pdword [pebp+psize*7]
  argpush 6,peax                       ;*len
  mov peax,pdword [pebp+psize*6]
  argpush 5,peax                       ;buf
  argpush 4,pdword [pebp+psize*5]      ;timeout
  argpush 3,pdword [pebp+psize*4]      ;flags
  argpush 2,pdword [pebp+psize*3]      ;sess
  argpush 1,pesi                       ;psess
end if

if used is64
  call pdword [r12+(psize*(2+RecvData))]
else
  call pdword [pedx+(psize*(2+RecvData))]
end if
ClearStackAftherCall 6

endRecvData:
errRecvData:
call dleave
ret psize*6

;------------------------------------------------------------------------------
;The CloseClientSession function shoudl shutdown the connection with the Client
;------------------------------------------------------------------------------
;---------
;Internal call, skip protocolsession check
;---------
CloseClientSession_internal:
call denter
MakeStack
jmp CloseClientSession_internal_jump
;---------
;C implementation
;---------
if used is64
    CloseClientSession_c:
    StartProc 2

    mov rcx,retCloseClientSession_c
    xchg qword [rsp],rcx
    jmp CloseClientSession_asm

    retCloseClientSession_c:
    sub rsp,2*psize
    EndProc 2
    jmp rcx
else
    CloseClientSession_c:
end if
;---------
;asm implementation
;---------
CloseClientSession_asm:
call denter
MakeStack

;check if Protocolsession and ClientSession are valid
mov pecx,errCloseClientSession
call checkProtocolSession

;--skip protocolSession check
CloseClientSession_internal_jump:
mov pecx,errCloseClientSession
call checkClientSession

mov peax,pdword [peax] ;Get BindedTohandle from the InternalClientStructure
;Get first psess of the BindedTohandle structure (=>Receiver protocol)
push 1
push peax
call GetDwordItem
mov pebx,peax ;(R/E)BX => Get first psess of the BindedTohandle structure (=>Receiver protocol)

;Get the internal protocol structure of the psess value
push pebx
call getInternalProtocolStructure
test peax,peax
jz errCloseClientSession

mov peax,pdword [peax+psize*4] ;Get modstruct of Internal ProtocolStructure

;call CloseClientSession
argpush 2,pdword [pebp+psize*3]      ;sess
argpush 1,pebx                       ;psess
if used is64
  call pdword [peax+(psize*(2+CloseClientSession))]
else
  call pdword [peax+(psize*(2+CloseClientSession))]
end if
ClearStackAftherCall 2

errCloseClientSession:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The SendData function prepares and sends data to the client.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    GetClientSessionInfo_c:
    StartProc 5

    mov rcx,retGetClientSessionInfo_c
    xchg qword [rsp],rcx
    jmp GetClientSessionInfo_asm

    retGetClientSessionInfo_c:
    sub rsp,5*psize
    EndProc 5
    jmp rcx
else
    GetClientSessionInfo_c:
end if
;---------
;asm implementation
;---------
GetClientSessionInfo_asm:
call denter

GetClientSessionInfo_next_psess:
mov pecx,errGetClientSessionInfo
call checkProtocolSession

mov pecx,errGetClientSessionInfo
call checkClientSession

xor ecx,ecx
inc ecx         ;decrement psess

;get next psess and get the modstruct of this psess
call FindNextOrPreviousPsess
test eax,eax
jz endGetClientSessionInfo

dec eax
test eax,eax
jnz endGetClientSessionInfo

if used is64
  cmp pdword [r12+(psize*(2+GetClientSessionInfo))],0
else
  cmp pdword [pedx+(psize*(2+GetClientSessionInfo))],0
end if
jne callNextGetClientSessionInfo

;The current protocol does not export RecvData, try the next psess
mov pdword [pebp+psize*2],pesi
jmp GetClientSessionInfo_next_psess

callNextGetClientSessionInfo:

MakeStack

if used isLin
  if used is64
    mov peax,pdword [pebp+psize*6]
    argpush 5,peax                       ;len
    argpush 4,pdword [pebp+psize*5]      ;buf
    argpush 3,pdword [pebp+psize*4]      ;type
    argpush 1,pesi                       ;psess
    argpush 2,pdword [pebp+psize*3]      ;sess
  end if
end if

;Make call to first BindToProtocolCallBack
if used is32 | used isWin
  mov peax,pdword [pebp+psize*6]
  argpush 5,peax                       ;len
  argpush 4,pdword [pebp+psize*5]      ;buf
  argpush 3,pdword [pebp+psize*4]      ;type
  argpush 2,pdword [pebp+psize*3]      ;sess
  argpush 1,pesi                       ;psess
end if

if used is64
  call pdword [r12+(psize*(2+GetClientSessionInfo))]
else
  call pdword [pedx+(psize*(2+GetClientSessionInfo))]
end if
ClearStackAftherCall 5


endGetClientSessionInfo:
errGetClientSessionInfo:
call dleave
ret psize*5