;------------------------------------------------------------------------------
;This function will read the port from sockaddr_in and convert it to litle-endian
;param:
;  sockaddr_in
;return: port of sockaddr_in
;-----------------------------------------------------------------------------
GetPort:
mov peax,pdword [pesp+psize] ;sockaddr_in
mov ax,word [peax+2] ;sin_port
xchg al, ah ;convert to litle-endian

;clear higher word of eax
and eax,0xFFFF
ret psize

;------------------------------------------------------------------------------
;This function will return the size of and the pointer to sin6_addr or sin_addr
;param:
;  pointer to {sizeof(sockaddr_in), sockaddr_in}
;return: (R/E)AX => pointer to sin6_addr or sin_addr
;        (R/E)CX => size of sin6_addr or sin_addr
;-----------------------------------------------------------------------------
GetSinAddr:
mov peax,pdword [pesp+psize] ;pointer to {sizeof(sockaddr_in), sockaddr_in}
mov pecx,pdword [peax]       ;sizeof(sockaddr_in)
add peax,psize               ;skip to pointer sockaddr_in or sockaddr6_in
shr ecx,2                    ;ECX = ECX / 4
add peax,pecx                ;seek to the sin_addr or sin6_addr structure
cmp ecx,4                    ;if it's IPv4, the shr instruction was enough to calculate the size of sin_addr
je endGetSinAddr

shl ecx,1                    ;calculate the size of sin6_addr

endGetSinAddr:
ret psize

;------------------------------------------------------------------------------
;This function will convert the address to a readable string
;param:
;  pointer to {sizeof(sockaddr_in), sockaddr_in}
;  output pointer
;  pointer to size of output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
ConvertAddress:
call denter

mov pedx,pdword [pebp+psize*5] ;flags
mov pesi,pdword [pebp+psize*4] ;size pointer

push pdword [pebp+psize*2]
call GetSinAddr
;(R/E)AX => pointer to sin6_addr or sin_addr
;(R/E)CX => size of sin6_addr or sin_addr

;test pedx,CSI_ASCII_STRING ;test on flag Ascii string
;jnz isAscii

;shr dword [pesi],1 ;length devide by 2 (since 1 wchar = 2 bytes)

;isAscii:
cmp ecx,4
je isIPv4     ;is sin_addr

if used is64
  cmp qword [peax],0
  jne isv6Address
else
  cmp dword [peax],0
  jne isv6Address

  cmp dword [peax+4],0
  jne isv6Address
end if

lea pedi,dword [peax+0xC]

cmp dword [peax+8],0xFFFF0000
cmove peax,pedi
je isIPv4     ;is sin_addr

isv6Address: 
mov pebx,IPv6AddressToString
mov ecx,40 ;ECX = 40
jmp ipVariablesSet


isIPv4:
mov pebx,IPv4AddressToString
shl ecx,2 ;ECX = 16 (4*4)
mov eax,dword [peax] ;get sib_addr from pointer

ipVariablesSet:
cmp dword [pesi],ecx
jae bufferIsSufficient

  mov dword [pesi],ecx  ;set requested buffer size
  xor eax,eax           ;clear return value
  jmp endConvertAddress

bufferIsSufficient:
push pedx                       ;flags
push pdword [pebp+psize*3]      ;output pointer
push peax                       ;sin_addr
call pebx                       ;call IPv6AddressToString or IPv4AddressToString

xor eax,eax
inc eax ;function succes

endConvertAddress:
call dleave
ret psize*4


;------------------------------------------------------------------------------
;This function will convert the IPv4 address to a readable string
;param:
;  sin_addr
;  output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
IPv4AddressToString:
call denter

mov eax,dword [pebp+psize*2]   ;sin_addr
mov ecx,4 ;4 bytes to process
xor edx,edx                    ;zero edx
mov pedi,pdword [pebp+psize*3] ;output pointer
mov bl,10                      ;divider
mov esi,dword [pebp+psize*4]   ;flags
cld
jmp processGroups

processGroupsWithDot:
  mov byte [pedi],'.' ;put a dot between every byte
  inc pedi

  ;test esi,CSI_ASCII_STRING
  ;jnz isOutputAscii3

  ;mov byte [pedi],0
  ;inc pedi

  ;isOutputAscii3:

processGroups:
  mov bh,ah ;save AH

  nextDigit:
    xor ah,ah    ;zero ah (=rest)
    div bl       ;devide by 10

    add ah,30h   ;add 0x30 (='0')

    push peax    ;push number on stack
    inc edx      ;increment value with pushed numbers on stack

    test al,al   ;if byte = zero, we are done
  jnz nextDigit

  ;set every calculated number on the string
  setChars:
    pop peax

    mov byte [pedi],ah
    inc pedi

    ;test esi,CSI_ASCII_STRING
    ;jnz isOutputAscii

    ;mov byte [pedi],0
    ;inc pedi

    ;isOutputAscii:

    dec edx
  jnz setChars

  mov ah,bh ;restore ah
  shr eax,8
loop processGroupsWithDot

xor ax,ax

;test esi,CSI_ASCII_STRING
;jnz isOutputAscii2
;stosw
;jmp endIPv4AddressToString

;isOutputAscii2:
stosb

endIPv4AddressToString:
call dleave
ret (psize*3)

;------------------------------------------------------------------------------
;This function will convert the IPv6 address to a readable string
;param:
;  pointer to sin6_addr
;  output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
IPv6AddressToString:
call denter

mov pesi,pdword [pebp+psize*2]   ;(R/E)SI => sin_addr
mov ecx,8 ;8 words to process
mov pedi,pdword [pebp+psize*3] ;output pointer
mov ebx,dword [pebp+psize*4]   ;flags in higher word of ebx
mov bx,0x10                    ;divider  in lower word of ebx
cld
jmp processGroupsv6

processGroupsv6WithDubbleDot:
  mov byte [pedi],':' ;put a dot between every byte
  inc pedi

  ;test ebx,CSI_ASCII_STRING
  ;jnz isOutputv6Ascii3

  ;xor al,al
  ;stosb

 ; isOutputv6Ascii3:

processGroupsv6:
  lodsw ;load word [esi] on ax
  xchg al, ah ;convert to litle-endian
  push pecx ;save ecx
  xor ecx,ecx ;clear ecx

  nextDigitv6:
    xor edx,edx  ;zero (e)dx
    div bx       ;devide by 0x10

    add dx,30h   ;add 0x30 (='0')
    cmp dx,'9'
    jna notAlphabetical

    add dx,7

  notAlphabetical:
    push pedx    ;push number on stack
    inc ecx      ;increment value with pushed numbers on stack

    test ax,ax   ;if byte = zero, we are done
  jnz nextDigitv6

  ;set every calculated number on the string
  setCharsv6:
    pop peax
    stosb

    ;test ebx,CSI_ASCII_STRING
    ;jnz isOutputv6Ascii2

    ;xor al,al
    ;stosb

    ;isOutputv6Ascii2:
  loop setCharsv6

  pop pecx ;restore ecx
loop processGroupsv6WithDubbleDot

xor al,al
stosb

;test ebx,CSI_ASCII_STRING
;jnz isOutputv6Ascii

;stosb

;isOutputv6Ascii:

call dleave
ret (psize*3)