;------------------------------------------------------------------------------
;This function will convert size of a file to a readable string
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;
;-----------------------------------------------------------------------------
if used convertBytesToString
  convertBytesToString:
  call denter

  mov pedi,pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  xor esi,esi

  if used is32
    ;copy Qword before div
    push  dword [pedi+4]
    push  dword [pedi]
    mov pedi,pesp
  end if

  if used is64

  xor rdx,rdx

  nextRound:
    mov rcx,rdi
    shr rcx,10 ;divide by 1024

    test rcx,rcx
    jz RoundedSize
    inc esi ;next size

    mov rdx,rdi
    mov rax,rcx
    shl rax,10
    xor rdx,rax

    mov rdi,rcx
  jmp nextRound
  else ;if used i32
    mov ebx,1024 ;to divide with
    xor edx,edx

  nextRound:
    cmp dword [pedi+4],0 ;check higher dword on zero
    jnz sizeIsAbove1024

    cmp dword [pedi],1024 ;check lower dword on <1024
    jbe RoundedSize

  sizeIsAbove1024:
    push pedi
    call DivQword
    inc esi
  jmp nextRound
  end if

  RoundedSize:

    mov eax,edx
    xor edx,edx
    mov ecx,100
    mul ecx

    shr eax,10 ;divide by 1024

    push peax

    mov ebx,10 ;base
    mov peax,pedi
    mov pedi,pdword [pebp+psize*3] ;output string
    if used is64
      call internalIntToStr
    else
      call internalIntQToStr
    end if


    pop peax

    test peax,peax
    jz noDecimalA


    push peax

    mov ax,','
    stosb

    pop peax
    push peax

    cmp peax,10
    JAE moreThanTenA

    mov ax,'0'
    stosb

    moreThanTenA:
    mov ebx,10 ;base
    pop peax
    call internalIntToStr

    noDecimalA:

    mov ecx,esi

    jecxz copyBytes

    mov eax,' kB'
    dec ecx
    jz copyDword

    mov eax,' MB'
    dec ecx
    jz copyDword

    mov eax,' GB'
    dec ecx
    jz copyDword

    mov eax,' TB'
    dec ecx
    jz copyDword

    mov eax,' PB'
    dec ecx
    jz copyDword

    mov eax,' EB'
    jmp copyDword

  copyBytes:
    mov eax,' byt'
    stosd ;copy ' byt'
    mov eax,'es'
    stosd ;copy ' es',0,0
    dec pedi
    jmp endConvertBytesToString

  copyDword:
    stosd ;copy

endConvertBytesToString:
  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UNICODE string
;output: 5GB (5.000.000.000 bytes)
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;-----------------------------------------------------------------------------
if used neatSizeFormat
  neatSizeFormat:
  call denter

  push pdword [pebp+psize*3] ;output string
  push pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  call convertBytesToStringW
  mov pedi,peax
  sub pedi,2 ;minus zero word

  mov eax, 0x00280020 ;' ('
  stosd

  push pedi ;for FormatBytes

  mov peax,pdword [pebp+psize*2]  ;pointer to Qword
  mov pebx,10                     ;base
  if used is32
    call internalIntQToStrW
  else
    call internalIntToStrW
  end if

  ;--push pedi already pushed
  call FormatBytes

  __findEndStr:
    cmp word [pedi],0
    jz __foundEndStr

    add pedi,2
  jmp __findEndStr

  __foundEndStr:

  mov eax,')'
  stosd

  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UTF-8/Asci string
;output: 5GB (5.000.000.000 bytes)
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;-----------------------------------------------------------------------------
if used neatSizeFormatA
  neatSizeFormatA:
  call denter

  push pdword [pebp+psize*3] ;output string
  push pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  call convertBytesToString
  mov pedi,peax
  dec pedi ;minus zero byte

  mov ax, ' ('
  stosw

  push pedi ;for FormatBytes

  mov peax,pdword [pebp+psize*2]  ;pointer to Qword
  mov pebx,10                     ;base
  if used is32
    call internalIntQToStr
  else
    call internalIntToStr
  end if

  xor al,al
  stosb
  dec pedi

  ;--push pedi already pushed
  call FormatBytesA

  __findEndStrA:
    cmp byte [pedi],0
    jz __foundEndStrA

    inc pedi
  jmp __findEndStrA

  __foundEndStrA:

  mov ax,')'
  stosw

  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will convert size of a file to a readable UNICODE string
;input
;  1:(x64) long lnog (x86) pointer to long long
;  2:output string
;
;output:
;  (R/E)AX => pointer to the end of the string
;
;-----------------------------------------------------------------------------
if used convertBytesToStringW
  convertBytesToStringW:
  call denter

  mov pedi,pdword [pebp+psize*2] ;(x64) long lnog (x86) pointer to long long
  xor esi,esi

  if used is32
    ;copy Qword before div
    push  dword [pedi+4]
    push  dword [pedi]
    mov pedi,pesp
  end if

  if used is64

  xor rdx,rdx

  nextRoundW:
    mov rcx,rdi
    shr rcx,10 ;divide by 1024

    test rcx,rcx
    jz RoundedSizeW
    inc esi ;next size

    mov rdx,rdi
    mov rax,rcx
    shl rax,10
    xor rdx,rax

    mov rdi,rcx

  jmp nextRoundW
  else ;if used i32
    mov ebx,1024 ;to divide with
    xor edx,edx

  nextRoundW:
    cmp dword [pedi+4],0 ;check higher dword on zero
    jnz sizeIsAbove1024W

    cmp dword [pedi],1024 ;check lower dword on <1024
    jbe RoundedSizeW

  sizeIsAbove1024W:
    push pedi
    call DivQword
    inc esi
  jmp nextRoundW
  end if

  RoundedSizeW:

    mov eax,edx
    xor edx,edx
    mov ecx,100
    mul ecx

    shr eax,10 ;divide by 1024

    push peax

    mov ebx,10 ;base
    mov peax,pedi
    mov pedi,pdword [pebp+psize*3] ;output string
    if used is64
      call internalIntToStrW
    else
      call internalIntQToStrW
    end if

    pop peax

    test peax,peax
    jz noDecimal

    push peax

    mov ax,','
    stosw

    pop peax
    push peax

    cmp peax,10
    JAE moreThanTen

    mov ax,'0'
    stosw

    moreThanTen:
    mov ebx,10 ;base
    pop peax
    call internalIntToStrW

    noDecimal:

    mov ecx,esi

    jecxz copyBytesW

    if used is32
      mov eax,0x006B0020 ;' k'
      mov edx,0x0062 ;'b'
    else
      mov rax,0x0062006B0020 ;' kB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x004D0020 ;' M'
      mov edx,0x0042 ;'B'
    else
      mov rax,0x0042004D0020 ;'MB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00470020 ;' G'
      ;mov edx,du 'B'
    else
      mov rax,0x004200470020 ;' GB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00540020 ;' T'
      ;mov edx,du 'B'
    else
      mov rax,0x004200540020 ;' TB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00500020 ;' P'
      ;mov edx,du 'B'
    else
      mov rax,0x004200500020 ;' PB'
    end if
    dec ecx
    jz copyTwoDwords

    if used is32
      mov eax,0x00450020 ;' E'
      ;mov edx,du 'B'
    else
      mov rax,0x004200450020 ;' EB'
    end if
    jmp copyTwoDwords

  copyBytesW:
    if used is32
      mov eax,0x00620020 ;' b'
      stosd ;copy ' b'
      mov eax,0x00740079 ;'yt'
      stosd ;copy 'yt'
      mov eax,0x00730065 ;'es'
      xor edx,edx
      jmp copyTwoDwords
    else
      mov rax,0x0074007900620020 ;' byt'
      stosq ;copy ' byt'
      mov eax,0x00730065 ;'es'
      stosd
      xor ax,ax
      stosw
      jmp endConvertBytesToStringW
    end if

  copyTwoDwords:
    if used is32
      stosd ;copy
      mov eax,edx
      stosd
    else
      stosq
    end if

endConvertBytesToStringW:
  mov peax,pedi
  call dleave
  ret psize*2
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers and add " bytes" behind the srting
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatBytes
FormatBytes:
  call denter

  ;add dots
  push pdword [pebp+psize*2]
  call FormatNumber

  call pushBytes__FormatBytes

    du ' bytes',0

pushBytes__FormatBytes:
  push pdword [pebp+psize*2]
  call strcatW

ReturnFormatBytes:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers and add " bytes" behind the srting as ASCI/UTF-8
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatBytesA
FormatBytesA:
  call denter

  ;add dots
  push pdword [pebp+psize*2]
  call FormatNumberA

  call pushBytes__FormatBytesA

    db ' bytes',0

pushBytes__FormatBytesA:
  push pdword [pebp+psize*2]
  call strcat

ReturnFormatBytesA:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatNumber
FormatNumber:
  call denter

  sub pesp,0x40 ;create output string on stack
  mov pedi,pesp
  add pedi,0x37; 0x1B

  ;get the last character of the input string
  mov pesi,pdword [pebp+psize*2] ;input string
  push pesi
  call strlenW
  add pesi,peax

  ;(R/E)CX => amount characters in input string
  mov pecx,peax
  shr pecx,1

  xor pedx,pedx
  STD ;reversed direction of string instructions

  ;copy zero character
  lodsw
  stosw

FormatLoop:
  ;add dot if three characters have been copied
  cmp edx,3
  jne NoPointYet

    mov ax,'.'
    stosw
    xor edx,edx

  NoPointYet:
    ;copy character
    inc edx
    lodsw
    stosw

loop FormatLoop
  cld ;clear reversed direction of string instructions

  add pedi,2

  ;copy the formated string to the output
  push pedi
  push pdword [pebp+psize*2]
  call strcpyW


ReturnFormatNumbers:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will add dots for every group of 3 numbers as ASCI/UTF-8
;input
;  2:input/output string
;
;-----------------------------------------------------------------------------
if used FormatNumberA
FormatNumberA:
  call denter

  sub pesp,0x40 ;create output string on stack
  mov pedi,pesp
  add pedi,0x37; 0x1B

  ;get the last character of the input string
  mov pesi,pdword [pebp+psize*2] ;input string
  push pesi
  call strlen
  add pesi,peax

  ;(R/E)CX => amount characters in input string
  mov pecx,peax
  ;-shr pecx,1

  xor pedx,pedx
  STD ;reversed direction of string instructions

  ;copy zero character
  lodsb
  stosb

FormatLoopA:
  ;add dot if three characters have been copied
  cmp edx,3
  jne NoPointYetA

    mov al,'.'
    stosb
    xor edx,edx

  NoPointYetA:
    ;copy character
    inc edx
    lodsb
    stosb

loop FormatLoopA
  cld ;clear reversed direction of string instructions

  inc pedi

  ;copy the formated string to the output
  push pedi
  push pdword [pebp+psize*2]
  call strcpy


ReturnFormatNumbersA:
  call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function combine names to the flags set and create a new string
;input:
;    1:list with flagnames. The list are string next to eachother, with a zero byte as separator
;    2:flags to print on screen

;
;output: (R/E)AX: pointer to memory with new unicode flags string
;

;------------------------------------------------------------------------------
if used getFlagsByName
getFlagsByName:
call denter

mov ecx,32
mov pesi,pdword [pebp+psize*2] ;flagNames
mov pebx,pdword [pebp+psize*3] ;flags

xor edx,edx ;count actuall set flags

push 0 ;last item

loopFeatures:
  shr ebx,1
  jnc nextBit ;jump if bit is not set

  test edx,edx
  jz dontPushSpace

  call pushSpace

    db ' ',0

pushSpace:
  push 0x1   ;ACSI
  
dontPushSpace:

  push pesi  ;ATTRIBUTE
  push 0x1   ;ACSI
  
  inc edx

nextBit:
  push pesi
  call strlen
  add pesi,peax
  inc pesi

  dec ecx
jnz loopFeatures

call CreateOneString

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function combine names to the flags set and create a new utf-8 string
;input:
;    1:list with flagnames. The list are string next to eachother, with a zero byte as separator
;    2:flags to print on screen
;    3:flags for CreateOneStringUtf8
;       0x1 The return value should be the size neccecery for this string
;    4:Buffer for CreateOneStringUtf8

;
;output: (R/E)AX: return value of CreateOneStringUtf8
;

;------------------------------------------------------------------------------
if used getFlagsByNameUtf8
getFlagsByNameUtf8:
call denter

mov ecx,32
mov pesi,pdword [pebp+psize*2] ;flagNames
mov pebx,pdword [pebp+psize*3] ;flags

xor edx,edx ;count actuall set flags

push 0 ;last item

loopFeatures__Utf8:
  shr ebx,1
  jnc nextBit__Utf8 ;jump if bit is not set

  test edx,edx
  jz dontPushSpace__Utf8

  call pushSpace__Utf8

    db ' ',0

pushSpace__Utf8:
  push 0x1   ;ACSI
  
dontPushSpace__Utf8:

  push pesi  ;ATTRIBUTE
  push 0x1   ;ACSI
  
  inc edx

nextBit__Utf8:
  push pesi
  call strlen
  add pesi,peax
  inc pesi

  dec ecx
jnz loopFeatures__Utf8
push pdword [pebp+psize*5] ;pointer
push pdword [pebp+psize*4] ;flags
call CreateOneStringUtf8

call dleave
ret psize*4
end if