
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

;This file contains functions that shorten the paramters that have to be passed

;------------------------------------------------------------------------------
;This function will call GetPluginData for an integer and return the integer on ECX
;input EAX: an integer as name (the flag GDPD_NAME_IS_INTEGER will be set)
;      ESI: flags:
;                 GET_FROM_LISTENSESSION   (0x0)
;                 GET_FROM_CLIENTSESSION   (0x1)
;                 GET_FROM_PROTOCOLSESSION (0x2)
;                 IS_LIST_POINTER          (0x4)
;                 NO_COPY_LIST             (0x8)
;                 MEMORY_ONLY              (0x10)

;output EAX:return value of GetPluginData
;output ECX:the requested integer
;------------------------------------------------------------------------------
if used GetPluginDataInt
GetPluginDataInt:
push pedx

;set size parameter
push psize
mov pecx,pesp

;make buffer
push 0
mov pedx,pesp

push GDPD_NAME_IS_INTEGER  ;flags

test esi,4  ;IS_LIST_POINTER
jz IsNotListPointer

or pdword [pesp],GDPD_IS_LIST  ;flags

IsNotListPointer:
test esi,8  ;NO_COPY_LIST
jz IsCopyList

or pdword [pesp],GDPD_NO_COPY_LIST  ;flags

IsCopyList:
test esi,0x10  ;MEMORY_ONLY
jz IsHD

or pdword [pesp],GDPD_MEMORY_ONLY  ;flags

IsHD:
;make a call to GetPluginData to receive the integer
push pecx                  ;len
push pedx                  ;data
push peax                  ;name

test esi,2
jz doClientSession

push 0                      ;sess
push 0                      ;lsess
jmp continueGetPluginDataInt

doClientSession:
test esi,1
jz doListenSession

push pdword [pebp+psize*3] ;sess
push 0                     ;lsess
jmp continueGetPluginDataInt

doListenSession:
push 0                     ;sess
push pdword [pebp+psize*3] ;lsess

continueGetPluginDataInt:
push pdword [pebp+psize*2] ;psess
call [GetPluginData]

pop pecx ;the integer
add pesp,psize ;delete the size parameter from stack

pop pedx
ret
end if

;------------------------------------------------------------------------------
;This function will call SavePluginData for an integer
;input EAX: an integer as name (the flag GDPD_NAME_IS_INTEGER will be set)
;      ESI: flags:
;                 GET_FROM_LISTENSESSION   (0x0)
;                 GET_FROM_CLIENTSESSION   (0x1)
;                 GET_FROM_PROTOCOLSESSION (0x2)
;                 IS_LIST_POINTER          (0x4)
;                 NO_COPY_LIST             (0x8)
;                 MEMORY_ONLY              (0x10)
;      EDX: Integer to save

;output EAX:return value of GetPluginData
;------------------------------------------------------------------------------
if used SavePluginDataInt
SavePluginDataInt:
push pedx

;make buffer
push pedx
mov pedx,pesp

;make a call to GetPluginData to receive the integer
push GDPD_NAME_IS_INTEGER  ;flags
test esi,4  ;IS_LIST_POINTER
jz IsNotListPointerSave

or pdword [pesp],GDPD_IS_LIST  ;flags

IsNotListPointerSave:
test esi,8  ;NO_COPY_LIST
jz IsCopyListSave

or pdword [pesp],GDPD_NO_COPY_LIST  ;flags

IsCopyListSave:
test esi,0x10  ;MEMORY_ONLY
jz IsHDSave

or pdword [pesp],GDPD_MEMORY_ONLY  ;flags

IsHDSave:
push psize                 ;len
push pedx                  ;data
push peax                  ;name

test esi,2
jz doClientSession_SavePluginDataInt

push 0                      ;sess
push 0                      ;lsess
jmp continueGetPluginDataInt_SavePluginDataInt

doClientSession_SavePluginDataInt:
test esi,1
jz doListenSession_SavePluginDataInt

push pdword [pebp+psize*3] ;sess
push 0                     ;lsess
jmp continueGetPluginDataInt_SavePluginDataInt

doListenSession_SavePluginDataInt:
push 0                     ;sess
push pdword [pebp+psize*3] ;lsess

continueGetPluginDataInt_SavePluginDataInt:
push pdword [pebp+psize*2] ;psess
call [SavePluginData]

add pesp,psize ;delete the buffer parameter from stack

pop pedx
ret
end if

;This file contains functions that shorten the paramters that have to be passed

;------------------------------------------------------------------------------
;This function will call GetPluginData for data and return the a pointer on ECX
;input 1: an integer as name (the flag GDPD_NAME_IS_INTEGER will be set)
;      2: flags:
;                 GET_FROM_LISTENSESSION   (0x0)
;                 GET_FROM_CLIENTSESSION   (0x1)
;                 GET_FROM_PROTOCOLSESSION (0x2)
;                 IS_LIST_POINTER          (0x4)
;                 NO_COPY_LIST             (0x8)
;                 MEMORY_ONLY              (0x10)
;      3: return pointer for size (can be zero)
;      pdword [pebp+psize*2]: protocolsession
;      pdword [pebp+psize*3]: listensession / clientsession

;output EAX:the requested pointer (or zero)
;------------------------------------------------------------------------------
if used GetPluginDataPointerSave
GetPluginDataPointerSave:
push pesi
push pecx
push pedx

mov peax,pdword [pesp+psize*4] ;set name
mov esi,dword [pesp+psize*5] ;set flags
call GetPluginDataPointer

mov pedx,pdword [pesp+psize*6] ;get return pointer for size
test pedx,pedx
jz endGetPluginDataPointerSave

  mov pdword [pedx],pecx ;set size

endGetPluginDataPointerSave:
pop pedx
pop pecx
pop pesi
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will call GetPluginData for data and return the a pointer on ECX
;input EAX: an integer as name (the flag GDPD_NAME_IS_INTEGER will be set)
;      ESI: flags:
;                 GET_FROM_LISTENSESSION   (0x0)
;                 GET_FROM_CLIENTSESSION   (0x1)
;                 GET_FROM_PROTOCOLSESSION (0x2)

;                 MEMORY_ONLY              (0x10)
;      pdword [pebp+psize*2]: protocolsession
;      pdword [pebp+psize*3]: listensession / clientsession

;output EAX:the requested pointer (or zero)
;output ECX:size of pointer
;------------------------------------------------------------------------------
if used GetPluginDataPointer
GetPluginDataPointer:
push pedx

;set size parameter
push 0
mov pecx,pesp

;-;make buffer
;-push 0
;-mov pedx,pesp

push GDPD_NAME_IS_INTEGER  ;flags

test esi,0x10  ;MEMORY_ONLY
jz IsHD2

or pdword [pesp],GDPD_MEMORY_ONLY  ;flags

IsHD2:
;make a call to GetPluginData to receive the integer
push pecx                  ;len
push 0                     ;data
push peax                  ;name

test esi,2
jz doClientSession2

push 0                      ;sess
push 0                      ;lsess
jmp continueGetPluginDataPointer

doClientSession2:
test esi,1
jz doListenSession2

push pdword [pebp+psize*3] ;sess
push 0                     ;lsess
jmp continueGetPluginDataPointer

doListenSession2:
push 0                     ;sess
push pdword [pebp+psize*3] ;lsess

continueGetPluginDataPointer:
push pdword [pebp+psize*2] ;psess
call [GetPluginData]

pop pecx ;get size

test pecx,pecx
jz endGetPluginDataPointer ;jump on error

push pecx ;get size
sub pesp,psize*7 ;restore the paramters from GetPluginData

push pecx ;size
call GetMemLocation ;get pointer to receive buffer
mov pedx,peax ;(R/E)DX => return pointer

mov pdword [pesp+psize*4],pedx ;set data parameter for GetPluginData
call [GetPluginData]

add pesp,psize*1 ;clear the size value
mov peax,pedx ;(R/E)AX => return value

endGetPluginDataPointer:
pop pedx
ret
end if

RELEASE_STRING_1 equ 0x1
RELEASE_STRING_2 equ 0x2
;------------------------------------------------------------------------------
;This function will combine two strings and release original if the flags are set
;input 1: input string 1
;      2: input string 2
;      3: flags:
;                 RELEASE_STRING_1   (0x1)
;                 RELEASE_STRING_2   (0x2)

;output EAX:pointer to new string
;------------------------------------------------------------------------------
if used combineStrings
combineStrings:
call denter

mov pesi,pdword [pebp+psize*2]  ;(R/E)SI => string 1
mov pebx,pdword [pebp+psize*3]  ;(R/E)BX => string 2

;calculate the size of the string 1
push pesi
call strlen
mov pecx,peax ;ECX = counter of string 1

;calculate the size of the string 2
push pebx
call strlen
mov pedx,peax ;ECX = counter of string 1

add peax,pecx ;(R/E)AX => size of both strings
inc peax      ;(R/E)AX => add zero terminating byte

;get enough memory to make bombine both strings
push peax
call GetMemLocation
mov pedi,peax ;(R/E)DI => pointer to combined string

if used is64
  mov r12,pedi ;R12 => begin of combined string
else
  push pedi ;save combined string
end if

rep movsb ;copy string 1

mov pesi,pebx ;(R/E)SI => string 2
mov pecx,pedx ;(R/E)CX => size of string 2
rep movsb ;copy string 2

xor al,al
stosb ;copy zero byte

mov ecx,dword [pebp+psize*4] ;ECX = flags
test ecx,RELEASE_STRING_1
jz dontReleaseString1

  ;free string 1
  push pdword [pebp+psize*2]  ;string 1
  call FreeMemLocation

dontReleaseString1:
test ecx,RELEASE_STRING_2
jz dontReleaseString2

  ;free string 2
  push pebx  ;(R/E)SI => string 2
  call FreeMemLocation

dontReleaseString2:
endCombineStrings:
if used is64
  mov peax,r12 ;(R/E)AX => begin of combined string
else
  pop peax ;restore combined string
end if

call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will get the address / port from client/server
;input 1: type parameter for function GetClientSessionInfo
;      pdword [pebp+psize*2]: protocolsession
;      pdword [pebp+psize*3]: listensession / clientsession

;output EAX:Pointer to the address / port
;------------------------------------------------------------------------------
GetClientSessionInfoShort:
push pebx ;save (R/E)BX

mov pebx,pdword [pesp+psize*2] ;(R/E)BX => type parameter for function GetClientSessionInfo

push 0 ;return value for len

push pesp ;&len
push 0    ;buf
push pebx ;type
push pdword [pebp+psize*3] ;sess
push pdword [pebp+psize*2] ;psess
call [GetClientSessionInfo]

cmp pdword [pesp],peax   ;check on zero
jz endGetClientSessionInfoShort

;allocate memory for the address / port
push pdword [pesp]
call GetMemLocation

push pesp ;&len
push peax ;buf
push pebx ;type
push pdword [pebp+psize*3] ;sess
push pdword [pebp+psize*2] ;psess

mov pebx,peax ;save pointer for memory on (R/E)BX

call [GetClientSessionInfo]

mov peax,pebx ;(R/E)AX => pointer with address / port

endGetClientSessionInfoShort:
add pesp,psize ;delete len value

pop pebx ;restore (R/E)BX
ret psize