
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will initialize the ClientSession (if TCP)
;------------------------------------------------------------------------------
HandleConnection:
StartProc 2 ;Set arguments on stack
call denter

xor eax,eax ;return value

mov pedx,pdword [pebp+psize*2]  ;psess

cmp [psess_TCP],pedx
jne endHandleConnection
;
;Handle TCP
;

mov esi,0x1 ;GET_FROM_CLIENTSESSION
mov eax,option_keepalive_interval
call GetPluginDataInt

push pecx ;push keepalive_interval (for function SetKeepAlive)


mov esi,0x1 ;GET_FROM_CLIENTSESSION
mov eax,option_keepalive_time
call GetPluginDataInt

push pecx ;push keepalive_time (for function SetKeepAlive)

mov esi,0x1 ;GET_FROM_CLIENTSESSION
mov eax,option_keepalive
call GetPluginDataInt
push pecx ;push keepalive boolean (for function SetKeepAlive)


mov pecx,pdword [pebp+psize*3]  ;sess

;find the data member of this protocol
push pdword [pecx+8]       ;ProtocolData structure
push pedx                  ;psess
call [FindProtocolDataStruct]
mov pedi,peax ;(R/E)DI => Datastruct
mov pesi,pdword [peax+psize*2] ;SOCKET


push pesi ;socket
call SetKeepAlive

;This event takes care of functions recv returning if there is no data (else it would be placed in a WAIT state)
push pesi                      ;SOCKET
call CreateSocketEvent
mov pebx,peax

push pesi
call CreateSocketEventClose
mov [pedi+psize*2 + EXTRASIZE_EVENT],peax  ;save event

;call the next HandleConnection function from the other protocols
push pecx       ;sess
push pedx       ;psess
call [CallNextHandleConnection]

push pdword [pedi+psize*2 + EXTRASIZE_EVENT] ;push Close Event

mov pedi,peax                   ;return value of the other HandleConnection

;------------------------------
;Clean up all the open handles
;------------------------------

push pesi
call SaflyCloseTCPSocket

push pebx
call CloseSocketEvent

call CloseSocketEvent  ;close close event

mov peax,pedi ;set return value back
endHandleConnection:
call dleave
EndProc 2
ret (FunctionReturnFactor*2)

;------------------------------------------------------------------------------
;This function will close the TCP connection  (or the listen socket)
;------------------------------------------------------------------------------
CloseClientSession_TCP:
StartProc 2 ;Set arguments on stack
call denter

mov pecx,pdword [pebp+psize*3]  ;sess

;find the data member of this protocol
push pdword [pecx+8]       ;ProtocolData structure
push pdword [pebp+psize*2] ;psess
call [FindProtocolDataStruct]

;--push pdword [peax+psize*2]
;--call SaflyCloseTCPSocket
;set close flag
lock or dword [peax+psize*2 + EXTRASIZE_FLAGS],EXTRASIZE_FLAG_STOP_SESSION

call dleave
ret (FunctionReturnFactor*2)

;------------------------------------------------------------------------------
;This function will receive data from the TCP connection
;------------------------------------------------------------------------------
RecvData_TCP:
StartProc 6 ;Set arguments on stack
call denter

xor eax,eax ;return value

;check if the psess value is TCP
mov pecx,pdword [pebp+(psize*2)] ;psess
cmp [psess_TCP],pecx
jne endRecvData_TCP

;find the data member of this protocol
mov pecx,pdword [pebp+psize*3]  ;lsess

push pdword [pecx+8]       ;ProtocolData structure
push pdword [pebp+psize*2] ;psess
call [FindProtocolDataStruct]
mov pebx,pdword [peax+psize*2] ;get the socket from the data member

;Should all communication be stopped?
test dword [peax+psize*2 + EXTRASIZE_FLAGS],EXTRASIZE_FLAG_STOP_SESSION
jnz errRecvData_TCP

push pdword [pebp+psize*7]  ;*len
push pdword [pebp+psize*6]  ;buf
push pdword [pebp+psize*5]  ;timeout
push pdword [pebp+psize*4]  ;flags
push pebx                   ;SOCKET
call RecvSocketData


endRecvData_TCP:
call dleave
EndProc 4
ret (FunctionReturnFactor*6)

errRecvData_TCP:
mov eax,CE_CONNECTION_CLOSED
jmp endRecvData_TCP

;------------------------------------------------------------------------------
;This function will send data to the client the TCP connection
;------------------------------------------------------------------------------
SendData_TCP:
StartProc 6 ;Set arguments on stack
call denter

xor eax,eax ;return value

;check if the psess value is TCP
mov pecx,pdword [pebp+(psize*2)] ;psess
cmp [psess_TCP],pecx
jne endSendData_TCP

;find the data member of this protocol
mov pecx,pdword [pebp+psize*3]  ;lsess

push pdword [pecx+8]       ;ProtocolData structure
push pdword [pebp+psize*2] ;psess
call [FindProtocolDataStruct]
mov pebx,pdword [peax+psize*2] ;get the socket from the data member

;Should all communication be stopped?
test dword [peax+psize*2 + EXTRASIZE_FLAGS],EXTRASIZE_FLAG_STOP_SESSION
jnz errSendData_TCP

;set flag RSF_USE_TIMEOUT if timeout != 0
cmp dword [pebp+psize*5],0
je noTimeOut

  or pdword [pebp+psize*4],RSF_USE_TIMEOUT

noTimeOut:
mov pesi,pdword [pebp+psize*6] ;(R/E)SI => buffer
mov pedi,pdword [pebp+psize*7] ;(R/E)DI => *len
mov ecx,dword [pedi]           ;ECX => len
mov dword [pedi],0             ;zeor len

push pebx                   ;SOCKET
call GetSendBufferSize  ;get the buffer size
cmp eax,ecx ;if buffer size > len
jae dontChangeBufferSize

  push pecx                   ;new buffer size
  push pebx                   ;SOCKET
  call SetSendBufferSize  ;get the buffer size

  push pebx                   ;SOCKET
  call GetSendBufferSize  ;get the buffer size

dontChangeBufferSize:
mov edx,eax ;EDX => internal buffer size

sendLoop:
  cmp ecx,edx ;if(len < internal buffer size) bytes to send=len
  cmovb edx,ecx

  push pedx ;create *len

  push pesp                   ;*len
  push pesi                   ;buf
  push pdword [pebp+psize*5]  ;timeout
  push pdword [pebp+psize*4]  ;flags
  push pebx                   ;SOCKET
  call SendSocketData
  test eax,eax
  jnz endSendData_TCP ;end if error

  pop peax ;get the bytes that where send
  add pesi,peax ;move the buffer by the bytes that are send
  add dword [pedi],eax ;add the bytes the *len value
  sub ecx,eax ;substract the sended bytes from the bytes to send

  ;mov peax,pdword [pesp-psize*4]  ;EAX => timeout (that was left from the previous call)
  ;mov pdword [pebp+psize*5],peax  ;set timeout parameter


;jz endSendLoop ;jump if the size to send is not zero yet

  ;push 100
  ;call sleep

;jmp sendLoop
jnz sendLoop ;jump if the size to send is not zero yet

endSendLoop:
xor eax,eax

endSendData_TCP:
call dleave
EndProc 4
ret (FunctionReturnFactor*6)

errSendData_TCP:
mov eax,CE_CONNECTION_CLOSED
jmp endSendData_TCP

;------------------------------------------------------------------------------
;This function will send inofrmation about the clientsession
;------------------------------------------------------------------------------
GetClientSessionInfo_TCP:
StartProc 5 ;Set arguments on stack
call denter

;find the data member of this protocol
mov pecx,pdword [pebp+psize*3]  ;lsess
mov pedx,pdword [pebp+psize*4]  ;flags
mov pedi,pdword [pebp+psize*5]  ;buff

push pdword [pecx+8]       ;ProtocolData structure
push pdword [pebp+psize*2] ;psess
call [FindProtocolDataStruct]
lea pesi,pdword [peax+psize*3]  ;(R/E)SI => pointer to {sizeof(sockaddr_in), sockaddr_in}

test dx,dx  ;CSI_CLIENT_ADDRESS
jz getClientAddress

dec dx      ;CSI_CLIENT_PORT
jz getClientPort

dec dx      ;CSI_LOCAL_ADDRESS
jz getLocalAddress

dec dx      ;CSI_LOCAL_PORT
jz getLocalPort

dec dx      ;CSI_CONNECTION_ALIVE
jz getConnectionAlive

xor eax,eax ;ret value = false
jmp endGetClientSessionInfo

;-----------------
;CSI_CONNECTION_ALIVE
;-----------------
getConnectionAlive:
;Should all communication be stopped?
test dword [peax+psize*2 + EXTRASIZE_FLAGS],EXTRASIZE_FLAG_STOP_SESSION
jz continueGetConnectionAlive

xor eax,eax
jmp setAliveBuffer

continueGetConnectionAlive:
xor ecx,ecx
mov pesi,pdword [pebp+psize*6] ;&len
cmp dword [pesi],4
cmovb peax,pecx
jb endGetClientSessionInfo

mov pecx,peax   ;PECX => ProtocolDataStruct

push pdword [peax+psize*2+EXTRASIZE_EVENT] ;close event
call IsWSAEventSignaled

xor eax,1 ;1=0 and 0=1
setAliveBuffer:
mov pesi,pdword [pebp+psize*5] ;&buf
mov dword [pesi],eax

xor eax,eax
inc eax
jmp endGetClientSessionInfo

;-----------------
;CSI_CLIENT_ADDRESS
;-----------------
getClientAddress:
push pesi    ;pointer to {sizeof(sockaddr_in), sockaddr_in}
call GetSinAddr
;(R/E)AX => pointer to sin6_addr or sin_addr
;(R/E)CX => size of sin6_addr or sin_addr
test edx,CSI_RAW_DATA
jnz copyRawData

push pedx                  ;flags
push pdword [pebp+psize*6] ;&len
push pedi                  ;buffer
push pesi                  ;pointer to {sizeof(sockaddr_in), sockaddr_in}
call ConvertAddress
jmp endGetClientSessionInfo

;-----------------
;CSI_CLIENT_PORT
;-----------------
getClientPort:
add pesi,psize ;seem to sockaddr_in structure
push pesi
call GetPort
mov ecx,2 ;port is 2 bytes
;(R/E)AX => port
;(R/E)CX => 2 bytes (size of port)
test edx,CSI_RAW_DATA
jnz copyRawDataNoPointer

mov pecx,pdword [pebp+psize*6]
cmp dword [pecx],4
jae sufficientBufferGetClientPort

mov dword [pecx],4
xor eax,eax
jmp endGetClientSessionInfo

sufficientBufferGetClientPort:
mov pecx,IntToStr
;mov pesi,IntToStrW
;test pedx,CSI_ASCII_STRING
;cmovz pecx,pesi

;convert port to string
push 10     ;divide by 10
push pedi   ;output buffer
push peax   ;port
call pecx

;return 1;
xor eax,eax
inc eax
jmp endGetClientSessionInfo

;-----------------
;CSI_LOCAL_ADDRESS
;-----------------
getLocalAddress:
add pesi,0x20+psize ;get sockadd_in from local address
jmp getClientAddress

;-----------------
;CSI_LOCAL_PORT
;-----------------
getLocalPort:
add pesi,0x20+psize ;get sockadd_in from local address
jmp getClientPort

copyRawDataNoPointer:
push peax
mov peax,pesp

copyRawData:
xchg pesi,peax ;(R/E)SI = source, (R/E)DI = destination

rep stosb ;copy the raw data

xor eax,eax
inc eax

endGetClientSessionInfo:
call dleave
EndProc 5
ret (FunctionReturnFactor*5)

errGetClientSessionInfo_TCP:
xor eax,eax
jmp endGetClientSessionInfo