#!/bin/bash
FASM_COMPILER="/bin/fasm"

pause() {
read -n 1 -r -p "Press any key to continue..."
}

writeInt() {
  local n
  local left
  local curByte
  local curByteHex
  printf "" > $1
  n=$1
  left=$2
  for i in {0..3}
  do
    curByte=$(( $left % 256))
    left=$(( $left / 256))
    printf -v curByteHex "%X" $curByte
    printf "\x${curByteHex}" >> $n
  done
}

clear
echo "Welcome to the installer of TV's server v3.20"
echo "Please check the settings below before continue"
echo "--"
echo "FASM compiler path: $FASM_COMPILER"
echo "--"
echo "Modify this file if not correct."
pause

while [ "$ID" != "1" ] && [ "$ID" != "2" ]; do
  echo "Would you like to install the 32-bit version or the 64-bit version?"
  echo "--"
  echo "1:32-bit"
  echo "2:64-bit"
  echo -n "Enter [1-2, default 2]: "

  read ID

  if [ "$ID" = "" ]; then
    ID="2"
  fi
done

if [ "$ID" = "1" ]; then
  ARCH="x86"
  BARCH="32"
  LD_ARG="-m elf_i386 -shared -o"
else
  ARCH="x64"
  BARCH="64"
  LD_ARG="-shared -o"
fi

isNum="false"
while [ "$isNum" = "false" ]; do
  echo -n "On which port should the control panel run? (1-65535, default 10): "
  read PORT
  if [ "$PORT" = "" ]; then
    PORT=10
  fi
  case ${PORT#[-+]} in
    (*[!0-9]*|'') isNum="false";;
    (*)           isNum="true";;
  esac

done

DIR="$( cd "$( dirname "$0" )" && pwd )"

FILES="${DIR}/*lin_$ARCH.asm"
for f in $FILES
do
  FNAME=${f:${#DIR}+1}
  echo "Compiling ${FNAME}..."
  $FASM_COMPILER "$f"
done


FILES="${DIR}/*$BARCH\_lin.asm"
for f in $FILES
do
  FNAME=${f:${#DIR}+1}
  echo "Compiling ${FNAME}..."
  $FASM_COMPILER "$f"
done

mkdir -p /etc/tvs-server-$ARCH
mkdir -p /etc/tvs-server-$ARCH/Modules
mkdir -p /etc/tvs-server-$ARCH/ListenSessions

FILES="${DIR}/*lin_$ARCH.o"
for f in $FILES
do
  SONAME="${f:0:-1}so"
  FNAME=${f:${#DIR}+1:-2}
  ld $LD_ARG $SONAME $f
  if [ "${FNAME:0:17}" = "tvs_server_plugin" ]; then
    printf $SONAME > /etc/tvs-server-${ARCH}/Modules/${FNAME}
  fi
done

mkdir -p /etc/tvs-server-$ARCH/ListenSessions/1
mkdir -p /etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_CONTROLPANEL
mkdir -p /etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_TCPIP
mkdir -p /etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_TCPIP/listById

printf "Control Panel" > /etc/tvs-server-$ARCH/ListenSessions/1/Name
printf "TVS_SOFTWARE_TCPIP\0TVS_SOFTWARE_CONTROLPANEL\0\0" > /etc/tvs-server-$ARCH/ListenSessions/1/BindedToProtocols

printf "\0\0\0\0" > /etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_TCPIP/ProtocolType
writeInt "/etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_TCPIP/listById/3" $PORT

writeInt "/etc/tvs-server-$ARCH/ListenSessions/1/TVS_SOFTWARE_CONTROLPANEL/ProtocolType" "196610"

./base$BARCH\_lin -i -b -l 1

echo "Done"
echo "You can now go to http://127.0.0.1:$PORT/ to configure TV's server"
pause
