
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

proc_dir db '/proc/',0
exe db '/exe',0
map db '/map_files/',0

RTLD_NOW equ 2

;Returns zero on error.
if used loadmodule | used loadmoduleA
   loadmodule:
   loadmoduleA:
   call denter
   alignStack

   argpush 2,RTLD_NOW
   argpush 1,pdword [pebp+(psize*2)]
   if used useStaticLinking
     call dlopen
   else
     call pdword [dlopen]
   end if
   ;test peax,peax
   ;jnz endLoadmodule

   ;call [dlerror]
   ;plog LOG_ERROR,"Error '", 0x11, peax, "'..."
   ;xor eax,eax

   ;endLoadmodule:
   call dleave
   ret (psize*1)
end if

;------------

if used unloadmodule
   unloadmodule:
   call denter
   alignStack

   argpush 1,pdword [pebp+(psize*2)]
   if used useStaticLinking
     call dlclose
   else
     call [dlclose]
   end if

   call dleave
   ret (psize*1)
end if

;------------

if used getfunc
   getfunc:
   call denter
   alignStack

   argpush 2,pdword [pebp+(psize*3)]
   argpush 1,pdword [pebp+(psize*2)]
   if used useStaticLinking
     call dlsym
   else
     call [dlsym]
   end if

   call dleave
    ret (psize*2)
end if

;------------------------------------------------------------------------------
;This function frees the pointer returned by GetModuleName
;input:
;    1:pointer returned by getModuleName


;output:
;       none
;------------------------------------------------------------------------------
if used FreeModuleName
  FreeModuleName:

  push pdword [pesp+psize]
  call FreeMemLocation

  ret psize
end if

;------------------------------------------------------------------------------
;This function will set the working directory to the directory the module is in
;input:
;    1:modhandle


;output:
;       Pointer with directory or zero
;------------------------------------------------------------------------------
SetModuleDirectoryAsCurrentDirectory:
call denter

push pdword [pebp+psize*2] ;module
call getDirectoryByModule

argpushsys 1,peax ;path
linuxCall 80,12  ;chdir

call dleave
ret psize

;------------------------------------------------------------------------------
;This function returns a pointer (allocated by GetMemLocation) with the directory
;input:
;    1:modhandle


;output:
;       Pointer with directory or zero
;------------------------------------------------------------------------------
if used getDirectoryByModule
getDirectoryByModule:
call denter

push pdword [pebp+psize*2] ;hmodule
call GetModuleName
mov pebx,peax

push pebx
call strlen

;add a end the of filename and add a quote there
lea pedi,pdword [pebx+peax]

xor ecx,ecx  ; zero ECX
dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
mov eax,'/'  ; byte to find with scasb => '/'

STD ;execute scasb function backward
repnz scasb   ;loop string until something else then a zero byte is found
CLD ;clear backward direction

;place a zero byte at the slash to get the executable direcotry
mov byte [pedi+2],0

endGetDirectoryByModule:
mov peax,pebx

call dleave
ret

failed:
push pebx
call FreeMemLocation

xor ebx,ebx
jmp endGetDirectoryByModule
end if

;------------------------------------------------------------------------------
;This function returns a pointer (possibly allocated by GetMemLocation) with the directory and filename
;Use FreeModuleName to free the pointer
;input:
;    1:modhandle


;output:
;       Pointer with directory and filename or zero
;------------------------------------------------------------------------------
if used GetModuleName
  GetModuleName:
  call denter

  mov pebx,pdword [pebp+psize*2] ;R/EBX => modulehandle we need to find path from

  push 4096
  call GetMemLocation
  mov pedi,peax

  ;if it is a pointer the memory we can do this
  push 4096
  push pedi
  push pebx
  call getModuleNameFromAddress
  test eax,eax
  cmovnz peax,pedi
  jnz endGetModuleName

  test pebx,pebx
  jz notFoundPath

  ;m,aybe it's a link_map ?
  push 4096
  push pedi
  push pdword [pebx] ;link_map->l_addr
  call getModuleNameFromAddress
  test eax,eax
  cmovnz peax,pedi
  jnz endGetModuleName

  notFoundPath:
  ;not found path
  push pedi
  call FreeMemLocation

  xor eax,eax

endGetModuleName:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function returns a filename is the memory address points to a symbol link
;input:
;    1:memory address that contains a file
;    2:buf
;    3:size


;output:
;       true or false
;------------------------------------------------------------------------------
getModuleNameFromAddress:
call denter

call getCurPid

mov pebx,pdword [pebp+psize*2] ;R/EBX => modulehandle we need to find path from

mov pesi,exe
mov pedi,map
test pebx,pebx
cmovnz pesi,pedi

push 0
push pesi
push 0x1
push peax
push 0x3
ppush proc_dir
push 0x1
call CreateOneStringA
mov pedi,peax ;R/EDI => path to exe of map_files

xor esi,esi
xor ecx,ecx

test pebx,pebx
jz getLink

;create return structure for startDirectoryEnumeration
push 1072
call GetMemLocation
mov pecx,peax ;R/ECX => return structure for startDirectoryEnumeration

push pecx ;return structure for startDirectoryEnumeration
push pedi ;path map_files
call startDirectoryEnumeration
mov pedx,peax  ;R/EDX => handle for directory loop
test peax,peax
jz notFoundDir

findMemlocation:
  test dword [pecx],0x10000
  jz nextFile

  lea pesi,pdword [pecx+4+(7*2)+4+8] ;filename

  findDash:
    lodsb

    test al,al
    jz nextFile

    cmp al,'-'
  jne findDash

  mov byte [pesi-1],0
  lea peax,pdword [pecx+4+(7*2)+4+8] ;filename

  push 0x10
  push peax
  call StrToInt

  cmp pebx,peax
  jc nextFile

  push 0x10
  push pesi
  call StrToInt
  cmp pebx,peax
  jnbe nextFile

  mov byte[pesi-1],'-'
  lea peax,pdword [pecx+4+(7*2)+4+8] ;filename

  push 0
  push peax
  push 0x1
  push pedi
  push 0x1 or 0x20000
  call CreateOneStringA
  mov pedi,peax ;R/EDI => path to exe of map_files
jmp getLink

nextFile:
  push pecx ;return structure for startDirectoryEnumeration
  push pedx ;handle for directory loop
  call nextDirectoryEnumeration
  test eax,eax
  jnz findMemlocation

push pedx ;handle for directory loop
call closeDirectoryEnumeration

xor edx,edx
jmp notFoundDir

getLink:
if used is32
  push ecx
end if
argpushsys 3,pdword [pebp+psize*4]    ;buf len
argpushsys 2,pdword [pebp+psize*3]    ;buf
argpushsys 1,pedi ;path
linuxCall 89,85  ;readlink

if used is32
  pop ecx
end if

test peax,peax
jl notFoundDir

xor edx,edx
inc edx

notFoundDir:
jpcxz dontFreeDirStructure

;free return structure for startDirectoryEnumeration
push pecx
call FreeMemLocation

dontFreeDirStructure:
;return link path
push pedi
call FreeMemLocation

mov eax,edx
call dleave
ret psize*3