
;TV's server version 3.00
;Copyright (c) 2014, Tycho Veltmeijer.
;All rights reserved.

STACK_SIZE equ (0x1000*5)
GUARD_PAGE_SIZE equ 0x1000

CLONE_VM        equ 0x00000100
CLONE_FS        equ 0x00000200
CLONE_FILES     equ 0x00000400
CLONE_SIGHAND   equ 0x00000800
CLONE_PTRACE    equ 0x00002000
CLONE_VFORK     equ 0x00004000
CLONE_PARENT    equ 0x00008000
CLONE_THREAD    equ 0x00010000
CLONE_NEWNS     equ 0x00020000  
CLONE_SYSVSEM   equ 0x00040000
CLONE_SETTLS    equ 0x00080000
CLONE_PARENT_SETTID     equ 0x00100000
CLONE_CHILD_CLEARTID    equ 0x00200000
CLONE_DETACHED          equ 0x00400000
CLONE_UNTRACED          equ 0x00800000
CLONE_CHILD_SETTID      equ 0x01000000  
CLONE_STOPPED           equ 0x02000000
SIGCHLD         equ 17

CLONE_IO        equ 0x80000000
CLONE_NEWIPC equ    0x08000000

SIGUSR1 equ 30
SIGUSR2 equ 31

WNOHANG equ 1

GS_FS_SIZE equ 0x1000

;------------------------------------------------------------------------------
;This function will kill the current thread
;return: Thread Id
;-----------------------------------------------------------------------------
if used exitThread
   exitThread:

   xor eax,eax
   linuxCall 60,1  ;exit
   int3 ;should never be reached
end if

;------------------------------------------------------------------------------
;This function will check if a process is still running
;     1: PID
;return: true or false
;-----------------------------------------------------------------------------
isProcessStillRunning:
call denter

argpushsys 4,0       ;*ru
argpushsys 3,WNOHANG ;options
argpushsys 2,0       ;*stat_addr
argpushsys 1,pdword [pebp+psize*2] ;PID
linuxCall 61,114     ;wait4

xor ecx,ecx
inc ecx
test peax,peax
cmovz peax,pecx
jz enIsProcessStillRunning

xor eax,eax

enIsProcessStillRunning:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will terminate a process
;     1: PID
;return: none
;-----------------------------------------------------------------------------
terminateProcess:
call denter

argpushsys 2,1                     ;signal
argpushsys 1,pdword [pebp+psize*2] ;PID
linuxCall 62,37                    ;kill

argpushsys 4,0       ;*ru
argpushsys 3,0       ;options
argpushsys 2,0       ;*stat_addr
argpushsys 1,pdword [pebp+psize*2] ;PID
linuxCall 61,114     ;wait4

call dleave
ret psize

;------------------------------------------------------------------------------
;This function is called when a function that is created by createthread returns
;return: Thread Id
;-----------------------------------------------------------------------------
if used __exitThread
__exitThread:

  if FunctionReturnFactor=0
    pop peax
  end if

  sub pesp,STACK_SIZE+GUARD_PAGE_SIZE+GS_FS_SIZE

  argpushsys 2,STACK_SIZE + GUARD_PAGE_SIZE + GS_FS_SIZE ;pdword [pesp]
  argpushsys 1,pesp
  linuxCall 11,91  ;munmap

  ;argpushsys 2,GS_FS_SIZE
  ;if used is64
  ;  argpushsys 1,pdword [fs:0]
  ;else
  ;  argpushsys 1,pdword [gs:0]
  ;end if
  ;linuxCall 11,91  ;munmap



   argpushsys 1,0
   linuxCall 60,1  ;exit
   int3 ;should not happen
 end if

;------------------------------------------------------------------------------
;This function will return the Thread id of the currrent thread
;return: Thread Id
;-----------------------------------------------------------------------------
if used getCurThreadid
   getCurThreadid:
   call denter

   linuxCall 186,224  ;sys_gettid

   call dleave
   ret
end if

;------------------------------------------------------------------------------
;This function will return the Thread id of the currrent thread
;return: Thread Id
;-----------------------------------------------------------------------------
if used getCurPid
   getCurPid:

   linuxCall 39,20  ;getpid

   ret
end if

;------------------------------------------------------------------------------
;This function will set the thread in WAIT position for x miliseconds
;param:
;  mili: the time in miliseconds to be in WAIT pos
;-----------------------------------------------------------------------------

if used sleep
   sleep:
   call denter

   ;divide by 1000 (1 second
   mov peax,pdword [pebp+(psize*2)]
   mov ecx,1000
   xor edx,edx
   div pecx

   ;fill in the struct
   push 0      ;nano seconds
   push peax  ;seconds
   mov pebx,pesp

   mov peax,pedx
   mov ecx,1000000
   mul pecx
   mov pdword [pesp+psize],peax

   argpushsys 2,0
   argpushsys 1,pebx
   linuxCall 35,162 ;sys_nanosleep
   ;ppush noErrorSleep
   ;call setErrNo

   ;call printError

   ;noErrorSleep:

   add pesp,psize*2


   call dleave
   ret psize
end if

;------------------------------------------------------------------------------
;This function return the time the system is running in miliseconds
;param:

;Return runtime in miliseconds
;-----------------------------------------------------------------------------
CLOCK_MONOTONIC equ 1
if used getTickCount
getTickCount:
call denter

;create __kernel_timespec
push 0      ;nano seconds
push 0      ;seconds

argpushsys 2,pesp
argpushsys 1,CLOCK_MONOTONIC
linuxCall 228,265 ;clock_gettime

;seconds to miliseconds
pop peax
xor edx,edx
mov ecx,1000
mul pecx
mov pebx,peax ;save milliseconds

;nanoseconds to miliseconds
pop peax
mov ecx,1000000
xor edx,edx
div ecx

add peax,pebx ;add seconds in milisecond and nanoseconds in milisecond to gether

call dleave
ret
end if 


;------------------------------------------------------------------------------
;This function will create a new thread with one parameter
;param:
;  base:  the base address to start the thread
;  param: the param r zero in failure
;-----------------------------------------------------------------------------
if used createthread
createthread:
call denter

push pebp

argpushsys 6,0 ;offset
argpushsys 5,-1 ;fd
argpushsys 4,MAP_PRIVATE or MAP_ANONYMOUS ;flags
argpushsys 3,PROT_READ or PROT_WRITE ;
argpushsys 2,STACK_SIZE + GUARD_PAGE_SIZE + GS_FS_SIZE ;length
argpushsys 1,0 ;addr
linuxCall 9,192  ;sys_mmap2

mov pedi,peax
pop pebp

add pedi,GS_FS_SIZE

;make guard page not readable/wirtable
argpushsys 3,PROT_NONE
argpushsys 2,GUARD_PAGE_SIZE
argpushsys 1,pedi
linuxCall 10,125  ;mprotect

if used is64
  mov r8,pedi
  sub r8,GS_FS_SIZE
  add r8,GS_FS_SIZE-psize
  mov qword [r8],r8
else
  mov esi,pedi
  sub esi,GS_FS_SIZE
  add esi,GS_FS_SIZE-psize;0x1000
  mov dword [esi],esi

  push 0;32 ;useable
  push 0 ;seg_not_present
  push 1 ;limit_in_pages
  push 0 ;read_exec_only
  push 0 ;contents
  push 0x51 ;1 ;seg_32bit
  push 0xFFFFF;0x10000 ;0x10000;0x1000 ;limit
  push esi ;base_addr
  push 12 ;peax ;entry_number
  mov esi,esp
end if

mov peax,pedi

add peax,STACK_SIZE+GUARD_PAGE_SIZE-(psize*3) ;start address, return address, param
mov pedi,pdword [pebp+psize*3] ;param
mov pdword [peax+psize*2],pedi ;set param
p2mov pdword [peax+psize],__exitThread,pecx ;set return address

mov pedi,pdword [pebp+psize*2] ;start address
mov pdword [peax],pedi ;set start address

if used is64
  argpushsys 5,r8 ;tls
  argpushsys 4,0  ;TID (Thread Id)
else
  argpushsys 5,0 ;TID (Thread Id)
  argpushsys 4,esi ;tls
end if
argpushsys 3,0 ;PID (Process Id)
argpushsys 2,peax ;stack
argpushsys 1,CLONE_THREAD or CLONE_SIGHAND or CLONE_FS or CLONE_VM or CLONE_FILES or CLONE_IO or CLONE_SETTLS
linuxCall 56,120 ;sys_clone

ppush cloneWasSuccessfull
call setErrNo

  ;error!!!!
  xor eax,eax
  jmp endCreatethread

cloneWasSuccessfull:
test peax,peax
jnz isCallersThread

if used New_Memory_management & used MemoryMultiThreaded
  call initNewPointerListInTEB
  call denter

    if used checkOnBufferOverflow
      argpush 1,pdword [pebp+psize*3+SizeUsedByInitNewPointerList_overflowcheck]  ;arg
      call pdword [pebp+psize*1+SizeUsedByInitNewPointerList_overflowcheck]  ;;function address
    else
      argpush 1,pdword [pebp+psize*3+SizeUsedByInitNewPointerList]  ;arg
      call pdword [pebp+psize*1+SizeUsedByInitNewPointerList]  ;function address
    end if

    call dleave
    call uninitNewPointerListInTEB

    add pesp,psize
    endCreatedThread
  ret (FunctionReturnFactor*1)
end if

  if used is64
    argpush 1,pdword [pesp+psize*2]  ;arg
  end if
  ret


isCallersThread:

endCreatethread:
call dleave
ret psize*2
end if

;
;
;
;;------------------------------------------------------------------------------
;;This function will clear the stack and end the thread
;;-----------------------------------------------------------------------------
;endThread:
;;delete stack
;sub pesp,psize
;
;;remove stack
;argpushsys 2,pdword [pesp]
;argpushsys 1,pesp
;linuxCall 11,91  ;munmap
;
;argpushsys 1,0
;linuxCall 60,1  ;exit
;
;;----

if used exitProcess
exitProcess:
argpushsys 1,pdword [pesp+psize]
linuxCall 60,1  ;exit
end if


macro endThread
{ common
argpushsys 1,0
linuxCall 60,1  ;exit
}