
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

longPath db '\\?\',0

if used getMimeTypeFromExtension
valueNameContentType db 'Content Type',0

;------------------------------------------------------------------------------
;This function will get a MIM
;input:
;    (R/E)AX:pointer with extension
;output:
;    (R/E)AX=> pointer with mime-type
;    (R/E)CX=> size of pointer
;------------------------------------------------------------------------------
getMimeTypeFromExtension:
push pedx

push valueNameContentType  ;valueName
push 0                     ;SubSubSubSubKey
push 0                     ;SubSubSubKey
push 0                     ;SubSubKey
push peax                  ;file extension
mov eax,HKEY_CLASSES_ROOT
push peax                  ;hKey
call ReadRegEx
test edx,edx
jz endGetMimeTypeFromExtension

;ReadRegEx failed, clear allocated memory
push peax
call FreeMemLocation

xor eax,eax  ;return zero

endGetMimeTypeFromExtension:
pop pedx
ret
end if

if used __FreeMimeTypeFromExtension_internal
__FreeMimeTypeFromExtension_internal:
jmp FreeMemLocation
end if

;------------------------------------------------------------------------------
;This function will open a file
;input:
;    1:pointer with filename
;    2:flags:
;       0x0     READ
;       0x1     WRITE&READ
;       0x2     CREATE_FILE
;       0x4     SHARE_READ_WHILE_WRITE
;       0x10000 filename is UTF-8
;       0x20000 don't make long path

;output:
;    (R/E)AX=> filehandle or -1 on failure
;------------------------------------------------------------------------------
if used openFileHandle
openFileHandle:
call denter
MakeStack

if used is32
  cmp [is9x],1
  je dontConvertString
end if

;replace '/' with '\'
push pdword [pebp+psize*2] ;filename
call fixPath_Utf8

mov peax,pdword [pebp+psize*3] ;EAX = flags
and eax,0x10000 ;UTF-8 flag
shr eax,12 ;make it 0x10 or 0x0
or eax,1   ;UTF-8 or ANSI

push 0
push pdword [pebp+psize*2] ;filename
push peax ;ANSI or UTF-8
test pdword [pebp+psize*3],0x20000
jne dontMakeLongPath
  push longPath
  push 0x1
 dontMakeLongPath:
call CreateOneString
mov pdword [pebp+psize*2],peax

dontConvertString:
mov edi,FILE_SHARE_READ ;dwShareMode

mov edx,0x80000000  ;GENERIC_READ
test pdword [pebp+psize*3],0x1 ;WRITE&READ
jz noWriteRightsRequired

  or edx,0x40000000 ;GENERIC_WRITE
  test pdword [pebp+psize*3],0x4 ;SHARE_READ_WHILE_WRITE
  jnz noWriteRightsRequired

  xor edi,edi ;dwShareMode

noWriteRightsRequired:
mov esi,OPEN_EXISTING ;ESI => parameter CreationDisposition
test pdword [pebp+psize*3],0x2 ;CREATE_FILE
jz noFileCreate

  mov esi,OPEN_ALWAYS

noFileCreate:
argpush 7,0                     ;hTemplateFile
argpush 6,0                     ;dwFlagsAndAttributes
argpush 5,pesi                  ;dwCreationDisposition
argpush 4,0                     ;lpSecurityAttributes
argpush 3,pedi                  ;dwShareMode
argpush 2,edx                   ;dwDesiredAccess
argpush 1,pdword [pebp+psize*2] ;filename
if used is32
  cmp [is9x],0
  je useWCreateFile

    call [CreateFileA]
    jmp usedACreateFile
end if

useWCreateFile:
call [CreateFileW]
push peax

push pdword [pebp+psize*2] ;filename
call FreeMemLocation

pop peax

usedACreateFile:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will return the size of a file (with limit of 4GiB on x86)
;input:
;    1:file handle

;output:
;    (R/E)AX => low integer with size

;------------------------------------------------------------------------------
if used getfilesize | (used getfilesizeLongLong & used is64)
getfilesize:
if used getfilesizeLongLong & used is64
  getfilesizeLongLong:
end if
call denter
MakeStack

if used is32
  ;renember: we cant use GetFileSizeEx in x86 if we want to support Windows NT
  push 0                    ;high word is not needed here (call getfilesizeLongLong in x86 mode for long long support)
  push dword [pebp+psize*2] ;file handle
  call [GetFileSize]
else
  ;x64 is supported since XP and GetFileSizeEx is supported since 2000, so no problem here.
  lea rdx,qword [pesp+psize*3] ;free spot on the stack for the return long long
  mov rcx,qword [pebp+psize*2] ;file handle
  call [GetFileSizeEx]

  mov rax,qword [pesp+psize*3] ;set file size on RAX
end if

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will return the size of the file
;input x86:
;    1:file handle
;    2:pointer to long long
;input x64:
;    1:file handle

;output x86:
;    EAX => low integer with size
;    [2] => high integer with size

;output x64:
;    RAX => size of file
;------------------------------------------------------------------------------
if used getfilesizeLongLong & used is32
getfilesizeLongLong:
call denter

mov pedi,dword [ebp+psize*3]  ;long long
add pedi,4                   ;high part of the long long

push pedi                ;the return pointer for the higher part of the size
push dword [pebp+psize*2] ;file handle
call [GetFileSize]

sub pedi,4 ;low part of the long long
stosd

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will read out a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to read

;output:
;    EAX => size read
;------------------------------------------------------------------------------
if used readfile
readfile:
call denter
MakeStack

if used is32
  push 0 ;make space for the return read size
  mov pedx,pesp
else
  lea r9,dword [rsp+psize*6] ;argument nNumberOfBytesToRead => pointer to free space on stack
end if

argpush 5,0  ;lpOverlapped
if used is32
  push edx ;nNumberOfBytesToRead
else
  ;argpush 4,r9 (already done)
end if
argpush 3,pdword [pebp+psize*4] ;size to read
argpush 2,pdword [pebp+psize*3] ;buffer
argpush 1,pdword [pebp+psize*2] ;file handle
call [ReadFile]

if used is32
  pop peax ;pop the read size
else
  mov eax,dword [rsp+psize*6] ;the read size
end if

call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will write to a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to write

;output:
;    EAX => size written
;------------------------------------------------------------------------------
if used writefile
writefile:
call denter
MakeStack

if used is32
  push 0 ;make space for the return write size
  mov pedx,pesp
else
  lea r9,dword [rsp+psize*6] ;argument lpNumberOfBytesWritten => pointer to free space on stack
end if

argpush 5,0  ;lpOverlapped
if used is32
  push edx ;nNumberOfBytesToRead
else
  ;argpush 4,r9 (already done)
end if
argpush 3,pdword [pebp+psize*4]  ;size to write
argpush 2,pdword [pebp+psize*3] ;buffer
argpush 1,pdword [pebp+psize*2] ;file handle
call [WriteFile]

if used is32
  pop peax ;pop the read size
else
  mov eax,dword [rsp+psize*6] ;the read size
end if

call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will set the file pointer to desired position
;input:
;    1:file handle
;    2:(x64)filepointer to seek to (x86)pointer to a long long with a filepointer to seek to

;output:
;    EAX => return value SetFilePointer (the low-order dword of the current file pointer
;------------------------------------------------------------------------------
if used seekToPointerFile
seekToPointerFile:
call denter
MakeStack

mov peax,pdword [pebp+psize*3] ;(x64)pointer to seek to (x86)pointer to a long long to seek to

if used is32
  ;SetFilePointerEx is avaidable since win 2000, so call SetFilePointer in 32-bit to support winNT
  push FILE_BEGIN            ;seek from begining
  push dword [peax+psize]    ;high part of the long long
  push dword [peax]          ;low part of the long long
  push dword [pebp+psize*2]  ;file handle
  call [SetFilePointer]
else
  ;x64 is avaidable since Win XP, so we are able to use SetFilePointerEx
  argpush 4,FILE_BEGIN            ;seek from begining
  argpush 3,0                     ;high part of the long long
  argpush 2,rax                   ;long long
  argpush 1,pdword [pebp+psize*2] ;file handle
  call [SetFilePointerEx]
end if


call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will set the file pointer to the end of the file
;input:
;    1:file handle

;output:
;    EAX => return value SetFilePointer (the low-order dword of the current file pointer
;------------------------------------------------------------------------------
if used seekToEndPointerFile
seekToEndPointerFile:
call denter
MakeStack

argpush 4,FILE_END              ;seek from begining
argpush 3,0                     ;high part of the long long
argpush 2,0                     ;low part of the long long
argpush 1,pdword [pebp+psize*2] ;file handle
call [SetFilePointer]

call dleave
ret psize*1
end if

;------------------------------------------------------------------------------
;This function will close a file handle
;input:
;    1:file handle

;output:
;    EAX => file handle
;------------------------------------------------------------------------------
if used closefile
closefile:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [CloseHandle]

mov peax,pdword [pebp+psize*2]

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will check if a directorie exist
;input:
;    1:path (UTF-8)

;output:
;       eax:0 (no directoty) 0x10:directory
;flags: Z: No directory  z: directory
;------------------------------------------------------------------------------
if used isDirectory
isDirectory:
call denter
MakeStack

if used is32
  cmp [is9x],0
  je UseWGetFileAttributes

    argpush 1,pdword [pebp+psize*2] ;path
    call [GetFileAttributesA]

    jmp usedAGetFileAttributes
end if

UseWGetFileAttributes:
;replace '/' with '\'
push pdword [pebp+psize*2] ;filename
call fixPath_Utf8

push 0
push pdword [pebp+psize*2]
push 0x11
push longPath
push 0x1
call CreateOneString
mov pebx,peax

argpush 1,pebx ;path
call [GetFileAttributesW]

push peax

push pebx
call FreeMemLocation

pop peax

usedAGetFileAttributes:
call dleave

cmp eax,-1
jz endIsDirectory

and eax,FILE_ATTRIBUTE_DIRECTORY

endIsDirectory:
ret psize
end if

;------------------------------------------------------------------------------
;This function start (and return an handle) to enumerate file(s) and directories in a directory
;input:
;    1:path (UTF-8)
;    2: return structure (total: 4+(7*2)+4+8+(260*4)=1070, rounded by 8=1072):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260] (UTF-8)



;output:
;       eax:HANDLE or zero
;------------------------------------------------------------------------------
if used startDirectoryEnumeration
directoryWildFlag db '*.*',0

startDirectoryEnumeration:
call denter
sub pesp,11*4 + 592 + 16 + 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;(R/E)SI => pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
xor eax,eax
if used is32
  mov ecx,592/4
  rep stosd
else
  mov ecx,592/8
  rep stosq
end if

MakeStack

if used is32
  cmp [is9x],0
  je useuseWFindFirstFile

    push 0 ;last item
    push directoryWildFlag
    push pdword [pebp+psize*2] ;path
    call combineStrings
    mov pedi,peax ;(R/E)SI => pointer to path + "*.*"

    argpush 2,pesi     ;LPWIN32_FIND_DATA
    argpush 1,pedi     ;path
    call [FindFirstFileA]
    jmp useAnsiFindFirstFile

  useuseWFindFirstFile:
end if

;replace '/' with '\'
push pdword [pebp+psize*2] ;path
call fixPath_Utf8

push 0 ;last item
push directoryWildFlag
push 0x1
push pdword [pebp+psize*2] ;path
push 0x11
push longPath
push 0x1
call CreateOneString
mov pedi,peax ;(R/E)SI => pointer to path + "*.*"

argpush 2,pesi     ;LPWIN32_FIND_DATA
argpush 1,pedi     ;path
call [FindFirstFileW]
useAnsiFindFirstFile:
mov pebx,peax  ;(R/E)BX => handle

;free temp path string
push pedi
call FreeMemLocation

inc pebx    ;-1 + 1 = 0
test pebx,pebx
cmovz eax,edx
jz endStartDirectoryEnumeration ;return if invalid handle
dec pebx  ;restore handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  cmp [is9x],0
  je copyWToUtf8

      mov ecx,65 ;=260 / 4
      rep movsd ;copy the filename
      jmp copyAtoA

  copyWToUtf8:
end if
push pesi
push pedi
call strcpyWToUtf8

copyAtoA:

;-if used is32
;-  mov ecx,65 ;=260 / 4
;-  rep movsd ;copy the filename
;-else
;-  mov ecx,32 ;=256 / 4
;-  rep movsq ;copy the first 256 bytes of the filename
;-  movsd     ;copy last 4 bytes
;-end if


mov peax,pebx ;return handle

endStartDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function return next next file in the directory
;input:
;    1:handle
;    2: return structure (total: 4+(7*2)+4+8+(260*4)=1070, rounded by 8=1072):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260] (UTF-8)



;output:
;       eax:true of false
;------------------------------------------------------------------------------
if used nextDirectoryEnumeration
nextDirectoryEnumeration:
call denter
sub pesp,(11*4) + 592 + 16 + 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
xor eax,eax
if used is32
  mov ecx,592/4
  rep stosd
else
  mov ecx,592/8
  rep stosq
end if

MakeStack

argpush 2,pesi                      ;LPWIN32_FIND_DATA
argpush 1,pdword [pebp+psize*2]     ;handle

if used is32
  cmp [is9x],0
  je UseWFindNextFileW

    call [FindNextFileA]
    jmp usedAFindNextFile

UseWFindNextFileW:
end if
call [FindNextFileW]
usedAFindNextFile:

test peax,peax
jz endNextDirectoryEnumeration ;return if invalid handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  cmp [is9x],0
  je copyWToUtf8_2

      mov ecx,65 ;=260 / 4
      rep movsd ;copy the filename
      jmp copyAtoA_2

  copyWToUtf8_2:
end if
push pesi
push pedi
call strcpyWToUtf8
copyAtoA_2:

;-if used is32
;-  mov ecx,65 ;=260 / 4
;-  rep movsd ;copy the filename
;-else
;-  mov ecx,32 ;=256 / 4
;-  rep movsq ;copy the first 256 bytes of the filename
;-  movsd     ;copy last 4 bytes
;-end if

xor eax,eax
inc eax

endNextDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function close the handle of the enumeration
;input:
;    1:handle


;output:
;       none
;------------------------------------------------------------------------------
if used closeDirectoryEnumeration
closeDirectoryEnumeration:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [FindClose]

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function returns a pointer (allocated by GetMemLocation) with the directory
;input:
;    1:modhandle


;output:
;       Pointer with directory or zero
;------------------------------------------------------------------------------
if used getDirectoryByModule
getDirectoryByModule:
call denter
MakeStack

push 261*2
call GetMemLocation
mov pebx,peax

;call GetModuleFileName to receive the path to thie application
argpush 3,260
argpush 2,pebx
argpush 1,pdword [pebp+psize*2]
call [GetModuleFileNameW]
test peax,peax
jz failed
cmp peax,260
jz failed

push 0
push pebx
push 0x2 or 0x20000       ;UTF-16 and free pointer
push 0                    ;buffer
push 0x0                  ;flags
call CreateOneStringUtf8
mov pebx,peax

push pebx
call strlen

;add a end the of filename and add a quote there
lea pedi,pdword [pebx+peax]

xor ecx,ecx  ; zero ECX
dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
mov eax,'\'  ; byte to find with scasb => '\\'

STD ;execute scasb function backward
repnz scasb   ;loop string until something else then a zero byte is found
CLD ;clear backward direction

;place a zero byte at the slash to get the executable direcotry
mov byte [pedi+2],0

endGetDirectoryByModule:
mov peax,pebx

call dleave
ret psize

failed:
push pebx
call FreeMemLocation

xor ebx,ebx
jmp endGetDirectoryByModule
end if

;------------------------------------------------------------------------------
;This function replaces '/' with '\' and returns if the path ens with an '\'
;input:
;    1:path (UTF-8)

;output:
;       R/Eax:True = end with '\', false if not
;------------------------------------------------------------------------------
if used fixPath_Utf8
fixPath_Utf8:
call denter

mov pesi,pdword [pebp+psize*2] ;Path UTF-8
mov pedi,pesi

;Windows paths use '\', not '/'

xor eax,eax 
changeForwardToBackwardSlash_fixPath:
  mov cl,al ;save last unicode point
  lodsb

  ;replace '\' with '\'
  cmp al,'/'
  jne checkDubbleBackslah

    mov al,'\'


    ;remove dubble backslash
    cmp cl,'\'
    je changeForwardToBackwardSlash_fixPath

  checkDubbleBackslah:
  cmp al,'\'
  jne nextChar__utf8

    ;remove dubble backslash
    cmp cl,'\'
    je changeForwardToBackwardSlash_fixPath

  nextChar__utf8:
  stosb

  test al,al
jnz changeForwardToBackwardSlash_fixPath

cmp cl,'\'
jne endFixPath

  inc eax

endFixPath:
call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function replaces '\' with '/' and returns if the path ens with an '\'
;input:
;    1:path (UTF-8)

;output:
;       R/Eax:True = end with '\', false if not
;------------------------------------------------------------------------------
if used fixPath_Web_Utf8
fixPath_Web_Utf8:
call denter

mov pesi,pdword [pebp+psize*2] ;Path UTF-8
mov pedi,pesi

;on the web paths use '/', not '\'

xor eax,eax 
changeBackwardToForwardSlash_fixPath:
  lodsb

  ;replace '\' with '\'
  cmp al,'\'
  jne nextChar__web_utf8

    mov al,'/'

  nextChar__web_utf8:
  stosb

  test al,al
jnz changeBackwardToForwardSlash_fixPath

cmp cl,'/'
jne endFixPath_Web_Utf8

  inc eax

endFixPath_Web_Utf8:
call dleave
ret psize
end if