
;TV's server version 3.20
;Copyright (c) 2014 - 2022, Tycho Veltmeijer.
;All rights reserved.

RIGHT_READ equ 0x1
RIGHT_WRITE equ 0x2
RIGHT_EXECUTE equ 0x4

include "include/memfunctions_modes.inc"
include 'include/memfunctions.inc'

if used initGetSmallestPortionPageAllocation
  smallestPortionOfPageSize dd 0
  if used is64
    dd 0 ;make it compatible with qword :)
  end if
end if

;------------------------------------------------------------------------------
;Enables data execution prevention on non-executable memory
;input:
;  none
;output: EAX: result of SetProcessDEPPloicy or nothing if the function is uncallable
;------------------------------------------------------------------------------
if used setDEBenabled
setDEBenabled:
  call denter
  MakeStack

  call pushKernel2
  db 'kernel32.dll',0
  pushKernel2:
  call loadmodule

  call pushSetProcessDEPPolicy
  db 'SetProcessDEPPolicy',0
  pushSetProcessDEPPolicy:
  push peax
  call getfunc
  test peax,peax
  jz endSetDEBenabled

    argpush 1,1 ;PROCESS_DEP_ENABLE
    call peax

    log LOG_MESSAGE, "Data execution prevention enabled..."

endSetDEBenabled:
  call dleave
  ret
end if

;------------------------------------------------------------------------------
;This will make a call to VirtualAlloc and VirtualQuery to determine the smallest
;peace of memory VirtualAlloc can allocate (useally ~4K). If Virtual Divider is
;it will also init the AllocatedMemory structure.
;input:
;  none
;output: unknown
;------------------------------------------------------------------------------
if used initGetSmallestPortionPageAllocation
  initGetSmallestPortionPageAllocation:
  cmp [smallestPortionOfPageSize],0
  jne endInitGetSmallestPortionPageAllocation

  push pesi

  push 10
  call GetMemLocationVirtual

  mov pesi,peax

  push 0
  push pesp
  push 1 ;read
  push pesi
  call doesVMLocationExistsEx

  push pesi
  call FreeMemLocationVirtual

  pop peax
  mov [smallestPortionOfPageSize],eax

  if used New_Memory_management & ~ used MemoryMultiThreaded
    call initNewPointerList
    mov [AllocatedMemorys],peax
  end if

  pop pesi

  endInitGetSmallestPortionPageAllocation:
  ret
end if

;------------------------------------------------------------------------------
;This sould be called when a new thread starts, a AllocatedMemory structure will
;be init and be placed on Stack, and pointer to stack will be placed on TEB.
;WARNING: Dont'forget to call uninitNewPointerListInTEB before exit the thread
;to clean up this structure from the stack!!

;INPUT:
;    NONE
;------------------------------------------------------------------------------
if used New_Memory_management & used MemoryMultiThreaded
  ;------------------------------------------------------------------------------
  ;Get UserReserved value (which is pointer to stack, created by initNewPointerListInTEB) from the TEB
  ;INPUT:
  ;    (R/A)EX: pointer from TEB to stack with AllocatedMemory structure
  ;------------------------------------------------------------------------------
  ;--set UserReserved from Teb
  if used GetUserReservedValueTeb
    GetUserReservedValueTeb:
    if used is64
      mov rax,qword [gs:qword 0x80]
      ret
    else
      call denter

      cmp [is9x],1
      jne getPebInWNT

      mov peax,dword [fs:0x1C]    ;set UserReserved in W9x
      jmp endGetUserReservedValueTeb

      getPebInWNT:
      ;Get PEB from TEB
      mov eax,dword [fs:0x30]
      cmp dword [eax+0xA4],4  ;check major version
      ja reserverValueIsReplaced

                         ;or should this be 0x0700? 0x06CC
      mov eax,dword [fs:0x06CC] ;get UserReserved in OS V4
      jmp endGetUserReservedValueTeb

      reserverValueIsReplaced:
      mov eax,dword [fs:0x44]   ;get UserReserved in OS V5+

      endGetUserReservedValueTeb:
      call dleave
      ret
    end if
  end if

  ;------------------------------------------------------------------------------
  ;Set UserReserved pointer on TEB
  ;INPUT:
  ;    (R/A)EX: value that need to be placed on the TEB. (pointer to stack with AllocatedMemory structure)
  ;------------------------------------------------------------------------------
  if used SetUserReservedValueTeb
    SetUserReservedValueTeb:
    if used is64
      mov qword [gs:qword 0x80],peax
      ret
    else
      push peax
      call denter
      cmp [is9x],1
      jne setPebInWNT

      mov peax,pdword [pebp+psize*1]
      mov dword [fs:0x1C],peax    ;set UserReserved in OS W9x
      jmp endSetUserReservedValueTeb

      setPebInWNT:
      ;Get PEB from TEB
      mov eax,dword [fs:0x30]

      cmp dword [eax+0xA4],4  ;check minor version
      ja reservedValueIsReplaced

      mov peax,pdword [pebp+psize*1]
      mov dword [fs:0x06CC],peax    ;set UserReserved in OS V4
      jmp endSetUserReservedValueTeb


      reservedValueIsReplaced:
      mov peax,pdword [pebp+psize*1]
      mov dword [fs:0x44],peax      ;set UserReserved in OS V5+

      endSetUserReservedValueTeb:
      call dleave
      pop peax
      ret
    end if
  end if
end if

if used GetMemLocation | used GetMemLocationVirtual
  ;------------------------------------------------------------------------------
  ;This function will make a new memory location with the required size by calling VirtualAlloc 1:mem location
  ;------------------------------------------------------------------------------
  GetMemLocationVirtual:
    call denter

  noDenter_GetMemLocation2:

    MakeStack

    mov peax,pdword [pebp+(psize*2)]

    argpush 4,0x04;PAGE_READWRITE (0x40 = PAGE_EXECUTE_READWRITE)
    argpush 3,0x1000
    argpush 2,peax
    argpush 1,0
    call [VirtualAlloc]

    call dleave
    ret (psize*1)
end if


if used FreeMemLocation | used FreeMemLocationVirtual
  ;------------------------------------------------------------------------------
  ;This function will free a memory location by calling VirtualFree 1:mem location
  ;-----------------------------------------------------------------------------
  FreeMemLocationVirtual:
  call denter
  MakeStack

  mov pecx,pdword [pebp+(psize*2)]
  jpcxz NothingToFree

  UnsafeFreeMemLocation:
  argpush 3,0x8000
  argpush 2,0
  argpush 1,pecx
  call [VirtualFree]

  NothingToFree:
  call dleave
  ret (psize*1)

end if

;------------------------------------------------------------------------------
;This function will check if the VM location exists and check the rights:
;int doesVMLocationExists(mem:void*, requiredRights:int)
;requiredRights can be:
;---RIGHT_READ 0x1
;---RIGHT_WRITE 0x2
;---RIGHT_EXECUTE 0x4
;------------------------------------------------------------------------------
if used doesVMLocationExists
   doesVMLocationExists:
   xor eax,eax
   xchg peax,pdword [pesp+psize*2] ;xchange requiredRights with zero (return value for size)
   xchg peax,pdword [pesp+psize]   ;xchange mem with requiredRights
   xchg peax,pdword [pesp]         ;xchange return address with mem
   push peax                       ;push return address
   jmp doesVMLocationExistsEx
end if

;------------------------------------------------------------------------------
;This function will check if the VM location exists and check the rights:
;int doesVMLocationExistsEx(mem:void*, requiredRights:int, *size:int)
;requiredRights can be:
;---RIGHT_READ 0x1
;---RIGHT_WRITE 0x2
;---RIGHT_EXECUTE 0x4
;------------------------------------------------------------------------------
if used doesVMLocationExistsEx
   doesVMLocationExistsEx:
   call denter
   MakeStack

   if used is64
      push (6*psize) ;MEMORY_BASIC_INFORMATION64
   else
      push (7*psize)
   end if
   call GetMemLocationVirtual
   mov pebx,peax

   if used is64
      mov r8,(6*psize) ;MEMORY_BASIC_INFORMATION64
   else
      push (7*psize)
   end if
   argpush 2,pebx    ;pointer to MEMORY_BASIC_INFORMATION64
   argpush 1,pdword [pebp+psize*2] ;address
   call [VirtualQuery]
   test peax,peax
   jz enddoesVMLocationExists

   mov eax,dword [pebx+psize*4] ;member state
   and eax,0x1000     ;if MEM_COMMIT
   jz enddoesVMLocationExists

   mov pecx,pdword [pebp+psize*4] ;return value size
   test pecx,pecx
   jz dontSetSizeParameter

   mov peax,pdword [pebx+psize*3] ;member RegionSize
   mov pdword [pecx],peax

   dontSetSizeParameter:
   mov eax,dword [pebx+(psize*4)+4] ;member protect

   ;First bit stands for NOACCES
   shr eax,1
   jc enddoesVMLocationExists

   mov ecx,eax
   xor eax,eax
   inc eax ;eax = RIGHT_READ

   ;next bit is READ_ONLY
   shr ecx,1
   jc enddoesVMLocationExists

   cmc ;CF != CF
   rcl eax,1 ;eax = RIGHT_READ |RIGHT_WRITE

   ;next bit is READWRITE
   shr ecx,1
   jc enddoesVMLocationExists

   ;next bit is WRITECOPY
   shr ecx,1
   jc enddoesVMLocationExists

   mov eax,0x4  ;eax = RIGHT_EXECUTE

   ;next bit is PAGE_EXECUTE
   shr ecx,1
   jc enddoesVMLocationExists

   or eax,1 ;eax = RIGHT_EXECUTE  or  RIGHT_READ

   ;next bit is PAGE_EXECUTE_READ
   shr ecx,1
   jc enddoesVMLocationExists

   or eax,2 ;eax = RIGHT_EXECUTE  or  RIGHT_READ | RIGHT_WRITE

   ;next bit is PAGE_EXECUTE_READWRITE
   shr ecx,1
   jc enddoesVMLocationExists

   ;next bit is PAGE_EXECUTE_WRITECOPY
   shr ecx,1
   jc enddoesVMLocationExists

   xor eax,eax

   enddoesVMLocationExists:
   ;do and operation with the required rights, if result is zero it does not meet the required rights
   and eax,dword [pebp+psize*3]
   jz returndoesVMLocationExists

   xor ecx,ecx ;retv
   xchg eax,ecx

   ;check if all the rights where met.
   cmp ecx,dword [pebp+psize*3]
   jne returndoesVMLocationExists

   mov peax,pdword [pebp+psize*2] ;yes the VM does exists and has the required rights

   returndoesVMLocationExists:
   push peax

   ;free MEMORY_BASIC_INFORMATION64
   push pebx
   call FreeMemLocationVirtual

   pop peax

   call dleave
   ret psize*3
end if